import numpy as np
import matplotlib.pylab as plt
import astropy_stark.mytemp0 as mt0
import scipy.stats as ss

twopi = np.pi*2
deg2rad = np.pi/180
planck = 6.626307004e-34
c      = 2.99792458e8
boltz  = 1.38064852e-23




def pytfb_sub(taus,embh,emdot,wavang, deginc, t0vin=-1, t0iin = -1, alpha_visc = -0.75, hxsch = 3.0, alpha_irad = -0.75, eta = 0.1, rlosch = 3.0, norm = 1, quick = 1, xstop = 15, udlnr = 0.01,thcent=1.0,thfwhm=0.2,oldsmooth = 0,newsmooth = 1,diagnose=0):
 tauhi  = taus[-1]
 taulo  = taus[0]
 ntaus  = np.shape(taus)[0]
 psis   = np.zeros(ntaus)
 
 #if you want a top hat response then its easy else do what you had before
 if (wavang < 0.0):
  idxinc = np.where((taus > thcent - thfwhm/2) & (taus < thcent + thfwhm/2))[0]
  psis[idxinc] = 1
 
 
 else:
 #either input desired reference temperature t0 at 1 light day, or calculate the
 #value appropriate for black body accretion disc given m and mdot 
  if (t0vin < 0):
   t0v   = mt0.tv0(embh,emdot)
  else:
   t0v   = t0vin
  
  if (t0iin < 0):
   t0i   = mt0.ti0(embh,emdot, eta = eta)
  else:
   t0i   = t0iin 
   
 
  #rlosch = 3.0
  #wavang = 4000.
  #hxsch  = 3.0
  
  
  nrad   = 10000
  
  #need to calculate 1/T^3 (r/r0)^alpha_irad x^2/wav^2 / (cosh(x) - 1) delta (tau - tau(r,theta)) dtau
  
  
  t0v2   = t0v*t0v
  t0v4   = t0v2*t0v2
  
  t0i2   = t0i*t0i
  t0i4   = t0i2*t0i2
  
  rsch   = 1.15821e-10*embh #scwarzchild radius in light days
  hx     = hxsch*rsch
  hx2    = hx*hx
  cosi   = np.cos(deginc*deg2rad)
  sini   = np.sin(deginc*deg2rad)
  hxci = hx*cosi
  hc_kwav = planck*c/wavang/1.e-10/boltz
  wavang2 = wavang*wavang
  rlold  = rlosch*rsch
  
  
  #print t0v,t0i
  
  #this estimate for the cutoff radius is based on the max radius of the
  #highest lag (re-arrange equation 3 in Starkey et al 2017)
  #should be ok but might exclude some significant low lags for a VERY hot, edge on disk
  rhisch = -1.* max((tauhi - hxci)/(1.+sini), tauhi) #if -ve then this is in light days
  #print rhisch
  if (rhisch < 0):
   rhild  = np.abs(rhisch)
  else:
   rhild  = rhisch*rsch
  
  
  
  
  
  av4   = alpha_visc*4 
  ai4   = alpha_irad*4 
  #now define radius grid be smart here.
  
  #use a cutoff x_stop to determine when to stop the radius grid
  dtau   = taus[1] - taus[0]
  rhilog = dtau
  rtemp = np.array([rhilog,10*rhilog]) 
  rtl   = np.log(rtemp)
  ttemp4 = t0v4*(rtemp)**av4 + t0i4*(rtemp)**ai4
  ttemp = np.sqrt(np.sqrt(ttemp4))
  ttl    = np.log(ttemp) 
  tstop  = hc_kwav/xstop
  grad = (rtl[1] - rtl[0]) / (ttl[1] - ttl[0])
  rhil = rtl[0] +  grad*( np.log(tstop) - ttl[0] )
  rhild   = np.exp(rhil)
  
  #rt4   = rtemp**4
  #ttemp4 = t0v4*(rtemp)**av4 + t0i4*(rtemp)**ai4
  #grad = (rt4[1] - rt4[0])/(ttemp4[1] - ttemp4[0])
  #t_stop4 = (hc_kwav/xstop)**4
  #rhild4 = rt4[0] + grad*(t_stop4 - ttemp4[0])
  #ttemp=ttemp4**0.25
  #xtemp = hc_kwav/ttemp
  #rhild4 = np.interp(t_stop4,ttemp4,rt4)
  #rhild = np.sqrt(np.sqrt(rhild4))
  #print rtemp
  #print ttemp
  #print xtemp
  #print xstop, tstop, rhild, grad, rtl, ttl
  #print rhild
  #print 'now defined a stopping radius e.g radius at which'
  #print 'temp give x =hc/(wav T) > xstop. xstop default is 15 now set up radius grid with this'
  #print 'upper radius BUT how can it be 2 light days because we need response at time delays',
  #print 'longer than this? FIX THIS' 
  #raw_input()
  #build grid
  rgridlog = np.exp( np.arange(np.log(rlold), np.log(rhilog), udlnr)[:-1] )#np.logspace(np.log(rlold),np.log(rhilog))
  rgridlin = np.arange(rhilog,rhild,rhilog)
  rgrid    = np.concatenate((rgridlog,rgridlin))
  nrad     = np.shape(rgrid)[0]
  dr       = rgrid[1:] - rgrid[:-1]
  
  
  
  
  #rgrid  = np.logspace(np.log10(rlold),np.log10(rhild),nrad)
  
 
  
  
  
  
  #calculate temperature at each radius grid
  tv4   = t0v4*(rgrid)**av4
  ti4   = t0i4*(rgrid)**ai4
  rir0b = ti4/t0i4 #this is just (r/r0)^alpha_irad
  ttot4 = tv4 + ti4
  ttot2 = np.sqrt(ttot4) 
  ttot  = np.sqrt(ttot2)
  ttot3 = ttot2*ttot
  
  #each point will have a time delay and a weighting append these to a list
  #for each point in the disc
  
  #loop of azimuths
  delsave = []
  wsave   = []
  if (diagnose == 1):
   azsave = []
   rsave = []
  
  if (quick == 1):
   for i in range(nrad-1):
    #quick way 
    ttotnow   = ttot[i]
    ttot3now  = ttot3[i]
    radlo = rgrid[i]
    radhi = rgrid[i+1]
    drad  = dr[i]#radhi - radlo
    
    #now azimuth grid
    azwidth = drad/radlo
    azgrid = np.arange(0.0, twopi,azwidth)
    naz = np.shape(azgrid)[0]
    nazsub1 = naz - 1
    azgrid_s = azgrid[1:] - azgrid[:-1]
    
    #print drad, radhi, azwidth,'sdfsd',radhi*azwidth
    #print radlo, radhi, azwidth, drad, naz,'sdfsf'
    
    raz   = np.random.uniform(radlo,radhi,nazsub1)   
    daz  = np.sqrt(raz*raz + hx2)
    az   = np.random.uniform(low=azgrid[:-1],high=azgrid[1:],size=nazsub1)#np.random.uniform(low=0,high=1,size=nazsub1)*azgrid_s + azgrid[:-1]#np.random.uniform(azgrid[:-1],azgrid[1:],1)
    caz  = np.cos(az)
    tdl  = hxci - raz*caz*sini + daz
    x    = hc_kwav/ttotnow#hc_kwav/ttot#hc_kwav/ttotnow
    
     
    #print radlo, x
    x2   = x*x
    #the radlo * drad *azwidth is the solid angle element
    #azwidth can be left off here as it is always the same
    weight = rir0b[i]/ttot3now * x2/wavang2/(np.cosh(x) - 1) * radlo*drad*azwidth                 #rir0b/ttot3 * x2/wavang2/(np.cosh(x) - 1) * radlo*drad*azwidth      #
    wsave.append([weight]*nazsub1)
    delsave.append(tdl)
    if (diagnose == 1):
     azsave.append(az)
     rsave.append(raz)
    
  else:
   for i in range(nrad-1):
    radlo = rgrid[i]
    radhi = rgrid[i+1]
    drad  = radhi - radlo
    
    #now azimuth grid
    azwidth = drad/radhi
    azgrid = np.arange(0.0, twopi,azwidth)
    naz = np.shape(azgrid)[0]
    nazsub1 = naz - 1
    azgrid_s = azgrid[1:] - azgrid[:-1]
    
    raz   = np.random.uniform(radlo,radhi,nazsub1)
    daz  = np.sqrt(raz*raz + hx2)
    az   = np.random.uniform(low=azgrid[:-1],high=azgrid[1:],size=nazsub1)#np.random.uniform(azgrid[:-1],azgrid[1:],1)
    caz  = np.cos(az)
    tdl  = hxci - raz*caz*sini + daz
    tv4   = t0v4*(raz)**av4 #modification for inner radius to the right (negligible)* (1 - np.sqrt(rlold/raz)) / (1 - np.sqrt(rlold/1))
    ti4   = t0i4*(raz)**ai4
    rir0b = ti4/t0i4 #this is just (r/r0)^alpha_irad
    ttot4 = tv4 + ti4
    ttot2 = np.sqrt(ttot4) 
    ttot  = np.sqrt(ttot2)
    ttot3 = ttot2*ttot
    x      = hc_kwav/ttot
    x2     = x*x
    weight = rir0b/ttot3 * x2/wavang2/(np.cosh(x) - 1) * radlo*drad*azwidth
    wsave.append(weight)
    delsave.append(tdl)
    if (diagnose == 1):
     azsave.append(az)
     rsave.append(raz)
    #print 'az',az
    #print ''
    #print 'azgrid',azgrid
   #tv4   = t0v4*(raz)**av4 #modification for inner radius to the right (negligible)* (1 - np.sqrt(rlold/raz)) / (1 - np.sqrt(rlold/1))
   #ti4   = t0i4*(raz)**ai4
   #rir0b = ti4/t0i4 #this is just (r/r0)^alpha_irad
   #ttot4 = tv4 + ti4
   #ttot2 = np.sqrt(ttot4) 
   #ttot  = np.sqrt(ttot2)
   #ttot3 = ttot2*ttot
   
   
   
   #introduce a lower threshold weight to abort the iterations (to save time)
   
  
  
  delsave = [item for sublist in delsave for item in sublist]
  wsave   = [item for sublist in wsave for item in sublist]
  if (diagnose == 1):
   rsave   = [item for sublist in rsave for item in sublist]
   azsave  = [item for sublist in azsave for item in sublist]



  delsave = np.array(delsave)
  wsave   = np.array(wsave)
  if (diagnose == 1):
   rsave   = np.array(rsave)
   azsave  = np.array(azsave)
  
  nds = np.shape(delsave)[0]
  
  
  
  #add smoothing function to approximate delta function
  #ntaus = np.shape(taus)[0]
  sigtau = 5*dtau/2
  sigtaulim = 3*sigtau
  if (oldsmooth == 1):
   sigtau2 = sigtau*sigtau
   
   #print('ok now it works but is slow')
   #print('need to find a numpy or scipy gaussian smoothing function')
   #print('ready made function wil be faster than anything I write down below')
   #print('was fine for fortran but python prefers ready made smoothing function')
   #print('google numpy or scipy gaussian filter 1d etc')
   #raw_input()
   #psis_2 = scipy.ndimage.filters.gaussian_filter1d(, sigtau, axis=-1, order=0, output=None, mode='reflect', cval=0.0, truncate=4.0) 
   psis = np.zeros(ntaus)
   for id in range(nds):
    delnow = delsave[id]
    idlo = int( max(1,np.floor((delnow - sigtaulim - taulo)/dtau)) )  
    #print 'dfsdf',ntaus, np.ceil((delnow + sigtaulim)/dtau), np.max(ntaus,np.ceil((delnow + sigtaulim)/dtau))
    idhi = int( min(ntaus,np.ceil((delnow + sigtaulim - taulo)/dtau)) )
    idinc = np.arange(idlo,idhi,1)
    #print id,delnow,idlo,idhi,taus[idlo],taus[idhi]
    tdelsubtau = delnow - taus[idinc]
    gtemp = np.exp(-0.5*tdelsubtau*tdelsubtau/sigtau2)
    gsum = np.sum(gtemp)
    wnow  = wsave[id]
    psis[idinc] = psis[idinc] + wnow*gtemp/gsum 
  
  
  elif (newsmooth ==1):  
   for i in range(ntaus):
    taunow = taus[i]
    idxlo = taunow - sigtaulim
    idxhi = taunow + sigtaulim
    idnow = np.where((delsave > idxlo) & (delsave < idxhi))[0]
    
    a = (taunow - delsave[idnow])/sigtau
    a2 = -a*a/2
    ea2 = np.exp(a2)
    #ea2sum = np.sum(ea2) dont seem to need ea2sum but put back in later if necessary
    ea2out= np.sum(wsave[idnow]*ea2)
    psis[i] = ea2out#/ea2sum
   
  #if no smoothing just sort the uneven array into ascending time order and release it (do not use psis as output in this case. BEST NOT TO USE THIS SETTING) 
  else:
   ida = np.argsort(delsave)
   delsave = delsave[ida]
   wsave = wsave[ida]
   if (diagnose == 1):
    azsave = azsave[ida]
    rsave = rsave[ida]   
  
  psis = np.nan_to_num((psis,0)) 
  psis = psis[0] 
 # for i in range(ntaus-1):
 #  taulo = taus[i]
 #  tauhi = taus[i+1]
 #  taulimlo = taulo - sigtaulim
 #  taulimhi = tauhi + sigtaulim
 #  taucent = (taulo + tauhi)/2
 #  idxsmooth = np.where((delsave >= taulimlo) & (delsave < taulimhi))[0]
 #  taunow    = delsave[idxsmooth]
 #  wnow      = wsave[idxsmooth]
 #  
 #  a = taunow - taucent
 #  b = sigtau
 #  aa_bb2 = a*a/b/b
 #  gauss     = np.exp(-aa_bb2/2)
 #  gsum      = np.sum(gauss)
 # 
 #
 #  #WONT SOME SURFACE ELEMENTS BE COUNTED TWICE AND SO BE INCLUDED IN MORE THAN ONE GAUSSIAN WEIGHTING? YES BUT I DONT THINK THIS MATTERS
 #  psis[i] = np.sum( wnow*gauss/gsum )
 #  
 #  
 #  #print taus[i], psis[i], ntaus
  #if norm == 1then normalise so max == 1
  if (norm == 1):
   #print psis[0]
   pt   = (psis - np.min(psis))
   psis = pt/np.max(pt)
 

 
 if (diagnose == 1):
   return(psis,delsave,wsave,rsave,azsave)
 else:
  return(psis)
#






#outputs are psis[ntau] to input set of time grid[ntau]
#make code to test and plot make nice plots
## 
#import timeit 
#
#wavref = 4000.
#embhref   = 1.e8
#emdotref  = 1.0
#degref    = 0.0
##
#t0v   = -1.6e4
#t0i   = -1.2e4
#taus   = np.arange(0.0,30.,0.1)
#ntau = np.shape(taus)[0]
##
##
##
##
##
##
#color=['r','b','cyan','purple','orange','gree','skyplue','magenta']
#fortrancomp = '/Users/ds207/Documents/standrews/sta/fort/fortcode/delaydist/tfbtest.dat'
#
#
#deg = [0.0,20.0,40.0,60.0,80.0]
#fig = plt.figure(111)
#ax1 = fig.add_subplot(111)
#idx = 0
#for dnow in deg:
# psis   = pytfb_sub(taus,embhref,emdotref,wavref, dnow,t0vin = t0v, t0iin = t0i, norm = 1,quick = 1,oldsmooth=0,newsmooth=1,diagnose = 0)
# ax1.plot(taus,psis,color=color[idx],label='deg = '+np.str(dnow))
# #calculate the mean
# mean = np.sum(taus*psis)/np.sum(psis)
# ax1.plot([mean,mean],[0.0,1.0],color=color[idx],label=None)
# idx = idx + 1
#ax1.set_xlabel('delay (days)')
#ax1.set_ylabel('response (arbitrary units)')
#if (fortrancomp != ''):
# fc = np.loadtxt(fortrancomp)
# ax1.plot(fc[:,0],fc[:,1],linewidth=2,color='k',label='fortran comp')
# mean = np.sum(fc[:,0]*fc[:,1])/np.sum(fc[:,1])
# ax1.plot([mean,mean],[0.0,1.0],color='k',linewidth=2,label=None)
#ax1.set_xlim([0,20.])
#ax1.legend()
#plt.savefig('plot_response_inc.pdf')
#











#other diagnostic tests testing smoothing function 



#start = timeit.default_timer()
#psis   = pytfb_sub(taus,embh,emdot,wavang, deginc,t0vin = t0v, t0iin = t0i, norm = 1,quick = 1)
#
#ds = psis[1]
#ws = psis[-1]
#
#ida = np.argsort(ds)
#ds = ds[ida]
#ws = ws[ida]
#nws = len(ws)
#
##weights = np.zeros((ntau,nws))
#
##tauds = np.zeros((ntau,nws))
##tauds[:,:] = taus
##tauds[:,:] = taus - ds
#
##dssub = ds - taus
##weights = np.exp(ds - taus)
#
#
#psis = psis[0]
#stop = timeit.default_timer()
#
#
#
#print 'run time', stop-start
# 
#fig = plt.figure()
#ax1 = fig.add_subplot(111)

#ax1.plot(ds,ws/ws.max())


#start = timeit.default_timer()
#psis   = pytfb_sub(taus,embh,emdot,wavang, deginc,t0vin = t0v, t0iin = t0i, norm = 1,quick = 0)
#
#ds = psis[1]
#ws = psis[-1]
#
#ida = np.argsort(ds)
#ds = ds[ida]
#ws = ws[ida]
#nws = len(ws)
#
##weights = np.zeros((ntau,nws))
#
##tauds = np.zeros((ntau,nws))
##tauds[:,:] = taus
##tauds[:,:] = taus - ds
#
##dssub = ds - taus
##weights = np.exp(ds - taus)
#
#
#psis = psis[0]
#stop = timeit.default_timer()
#print 'run time', stop-start
#
#fig = plt.figure()
#ax1 = fig.add_subplot(111)
#ax1.plot(taus,psis,label='quick off old smooth')
#ax1.plot(ds,ws/ws.max(),label='quick off no smooth')
#










#experiment 2 
#
#psis   = pytfb_sub(taus,embh,emdot,wavang, deginc,t0vin = t0v, t0iin = t0i, norm = 1,quick = 0)
#ds = psis[1]
#ws = psis[-1]
#ida = np.argsort(ds)
#ds = ds[ida]
#ws = ws[ida]
#nws = len(ws)
#
#datnew = ss.binned_statistic(ds,ws,statistic='mean',bins=taus)
#a = datnew[0]
#a = np.insert(a,0,0)
#ax1.plot(taus,a)
#
#plt.savefig('pytfbplottest_quick.pdf')

#scipy.stats.binned_statistic(x, values, statistic='mean', bins=10, range=None)
#try making histogram of points and sampling mean from each bin



















#experiment 3 
#
#psis   = pytfb_sub(taus,embh,emdot,wavang, deginc,t0vin = t0v, t0iin = t0i, norm = 1,quick = 0)
#ds = psis[1]
#ws = psis[-1]
#ida = np.argsort(ds)
#ds = np.array(ds[ida])
#ws = np.array(ws[ida])
#nws = len(ws)
#dtau = taus[1] - taus[0]
#sigtau = 5.*dtau
#ntaus = np.shape(taus)[0]
#psinew = np.zeros(ntaus)
#
##taunew = np.insert(np.arange(-10.0
#for i in range(ntaus):
# taunow = taus[i]
# a = (taunow - ds)/sigtau
# a2 = -a*a/2
# ea2 = np.exp(a2)
# ea2sum = np.sum(ea2)
# ea2sum = np.sum(ws*ea2/ea2sum)
# psinew[i] = ea2sum
# 
#ax1.plot(taus,psinew/psinew.max(),label='quick off new smooth')
#ax1.legend()
#plt.savefig('pytfbplottest_quick.pdf')
#
#scipy.stats.binned_statistic(x, values, statistic='mean', bins=10, range=None)
#try making histogram of points and sampling mean from each bin






##experiment 3
#fig = plt.figure()
#ax1 = fig.add_subplot(111)
#
#print 'experiment 1 old smoothing quick = 1'
#start = timeit.default_timer()
#psis   = pytfb_sub(taus,embh,emdot,wavang, deginc,t0vin = t0v, t0iin = t0i, norm = 1,quick = 1,oldsmooth=1,newsmooth=0,diagnose = 1)
#stop = timeit.default_timer()
#print 'run time', stop-start
#ax1.plot(taus,psis[0]/psis[0].max(),label='experiment 1 old smoothing quick = 1')
#
#print 'experiment 2 no smoothing quick = 1'
#start = timeit.default_timer()
#psis   = pytfb_sub(taus,embh,emdot,wavang, deginc,t0vin = t0v, t0iin = t0i, norm = 1,quick = 1,oldsmooth=0,newsmooth=0, diagnose = 1)
#stop = timeit.default_timer()
#print 'run time', stop-start
#ax1.plot(psis[1],psis[2]/psis[2].max(),label='experiment 2 no smoothing quick = 1')
#
#
#
#print 'experiment 3 new smoothing quick = 1'
#start = timeit.default_timer()
#psis   = pytfb_sub(taus,embh,emdot,wavang, deginc,t0vin = t0v, t0iin = t0i, norm = 1,quick = 1,oldsmooth=0,newsmooth=1,diagnose = 0)
#stop = timeit.default_timer()
#print 'run time', stop-start
#ax1.plot(taus,psis/psis.max(),label='experiment 3 new smoothing quick = 1')
#
#ax1.legend()
#plt.savefig('pytfbplottest_quick.pdf')
#
#
#
#
###plot the azimuth and radius grid to identify the source of hte hump
#rsave = psis[3]
#azsave = psis[4]
#fig = plt.figure()
#ax1 = fig.add_subplot(311)
#ax1.hist(rsave,bins = 200)
#ax1 = fig.add_subplot(312)
#ax1.hist(azsave,bins = 200)
#ax1 = fig.add_subplot(313)
#ax1.hist(psis[1],bins = 200)
##plt.show()
#plt.savefig('pytfbplottest_hist.pdf')
#
#
##fig = plt.figure()
##ax1 = fig.add_subplot(111)
##ax1.scatter(rsave*np.cos(azsave),rsave*np.sin(azsave))
##plt.savefig('pytfbplottest_scatter.pdf')
#















import numpy as np
import mylcgen as mlc
import matplotlib.gridspec as gridspec
import matplotlib.pylab as plt
import scipy.signal as ss







def ccf_simp(lc1_in,lc2_in,dt=1.0,r_crit = 0.8,laglim='auto'):

 lc1 = np.array(lc1_in,dtype=float)
 lc2 = np.array(lc2_in,dtype=float)
 
 #start of function
 #dt = 0.2 #time grid resolution
 
 #only select overlapping periods to take part in time series analysis
 tmin = max(np.min(lc1[:,0]),np.min(lc2[:,0]))#min(np.min(lc1[:,0]),np.min(lc2[:,0]))
 tmax = min(np.max(lc1[:,0]),np.max(lc2[:,0]))#max(np.max(lc1[:,0]),np.max(lc2[:,0]))
 
 
 #interpolate onto regular time grid
 tg  = np.arange(tmin,tmax+dt,dt)
 yg1 = np.interp(tg,lc1[:,0],lc1[:,1])
 yg2 = np.interp(tg,lc2[:,0],lc2[:,1])
 
 yg1_mean = np.mean(yg1)
 yg1_sd   = np.std(yg1)
 yg1 = (yg1 - yg1_mean)/yg1_sd

 yg2_mean = np.mean(yg2)
 yg2_sd   = np.std(yg2) 
 yg2 = (yg2 - yg2_mean)/yg2_sd
 
 
 #compute ccf
 ccf = ss.correlate(yg1,yg2)
 nccf = np.shape(ccf)[0]
 tccf = (np.arange(nccf) - nccf/2)*dt
 
 if (laglim != 'auto'):
  idccf = np.where((tccf > laglim[0]) & (tccf < laglim[1]))[0]
  ccf = ccf[idccf]
  tccf = tccf[idccf]
  nccf = np.shape(ccf)[0]
  #for it in range(nccf):
  # print(it,tccf[it], ccf[it])
  #print(np.argmax(ccf))
 
 
 ccf_max = np.max(ccf)
 
 #Peterson, Edelson 2018 definition of ccf inclusion threshold (>0.8 ccfmax)
 ccf_crit = r_crit*ccf_max
 idinc = np.where(ccf > ccf_crit)[0] 
 
 #weighted mean lag
 #lagmean = np.sum(ccf[idinc]*tccf[idinc])/np.sum(ccf[idinc])
 #laglo = tccf[idinc[0]]
 #laghi = tccf[idinc[-1]]
 
 
 #lagpeak
 idmax = np.argmax(ccf)
 
 try:
  idhi = idmax + np.where(ccf[idmax:] < ccf_crit)[0][0]
 except:
  print('ccf upper lag limit goes off the end of the grid. Re run with higher upper laglim')
  idhi = nccf-1
 try:
  idlo = np.where(ccf[:idmax] < ccf_crit)[0][-1]
 except:
  idlo = 0
  print('ccf lower lag limit goes off the lower end of the grid. Re run with lower lower laglim')
 
 lagpeak = tccf[idmax]
 laglo   = tccf[idlo]
 laghi   = tccf[idhi]
 
 print ('ccf low, peak, hi ',laglo,lagpeak,laghi)
 return(tccf,ccf,laglo,lagpeak,laghi,idlo,idmax,idhi)







#frrss ccf sampler

def ccf_frrss(lc1,lc2,dt=1.0,fraction_rss=0.8,nsim = 500,
centroid_frac = 0.8,flux_randomisation=0):
#
 n1 = np.shape(lc1)[0]
 n2 = np.shape(lc2)[0]
 
 n1pick = np.int(fraction_rss*n1)
 n2pick = np.int(fraction_rss*n2)

 #only select overlapping periods to take part in time series analysis
 tmin = max(np.min(lc1[:,0]),np.min(lc2[:,0]))#min(np.min(lc1[:,0]),np.min(lc2[:,0]))
 tmax = min(np.max(lc1[:,0]),np.max(lc2[:,0]))#max(np.max(lc1[:,0]),np.max(lc2[:,0]))
 tgrid = np.arange(tmin,tmax+dt,dt)

 print ('tmin max',tmin,tmax,dt)
 lagpeak = []
 ccfpeak = []
 ccf_save = []
 lagcent = []
 for i in range(nsim):
  id1 = np.sort(np.random.choice(np.arange(n1),size=n1pick,replace=True))
  id2 = np.sort(np.random.choice(np.arange(n2),size=n2pick,replace=True))
  #print(id1)
  
  if (flux_randomisation==1):
   noise1 = np.random.randn(n1pick)*lc1[id1,2]
   noise2 = np.random.randn(n2pick)*lc2[id2,2]
  else:
   noise1 = np.zeros(n1pick)
   noise2 = np.zeros(n2pick)
   
  x1 = np.interp(tgrid,lc1[id1,0],lc1[id1,1]+noise1)
  x2 = np.interp(tgrid,lc2[id2,0],lc2[id2,1]+noise2)
  x1mean = np.mean(x1)
  x1sd   = np.std(x1)
  x2mean = np.mean(x2)
  x2sd   = np.std(x2) 
  #compute ccf
  #ccf =np.correlate((x2-x2mean)/x2sd,(x1-x1mean)/x1sd,mode='full')#ss.correlate((x1-x1mean)/x1sd,(x2-x2mean)/x2sd)
  x1_new = (x1-x1mean)/x1sd
  x2_new = (x2-x2mean)/x2sd
  auto1 = np.sum(x1_new**2)#np.abs(np.correlate(x1_new,x2_new).max())
  #auto2 = np.correlate((x2-x2mean)/x2sd,(x2-x2mean)/x2sd).max()
  ccf = np.correlate(x1_new,x2_new,mode='full')/auto1
  
  if (i == 0):
   nccf = np.shape(ccf)[0]
   tccf = (np.arange(nccf) - np.floor(nccf/2))*dt
   tmax = tccf[-1]
   tmin = tccf[0]
   lolim = tmin/2
   hilim = tmax/2
   idinc = np.where((tccf < hilim) & (tccf > lolim))[0]
   lag_ccf = tccf[idinc]
   
  idpeak = np.argmax(ccf[idinc])
  ccfp = ccf[idinc][idpeak]
  tpeak = tccf[idinc][idpeak]
  ccf_save.append(ccf[idinc])
  lagpeak.append(tpeak)
  ccfpeak.append(ccfp)
  
  ccfinc = ccf[idinc]
  laginc = tccf[idinc]
  
  idcent = np.where(ccfinc > centroid_frac*ccfp)[0]
  lagcent.append( np.sum(laginc[idcent]*ccfinc[idcent])/np.sum(ccfinc[idcent]) )
  
  
  #fig = plt.figure()
  #ax1 = fig.add_subplot(211)
  #ax1.plot(tgrid,x1,label='drive',color='k')
  #ax1.plot(tgrid,x2,label='echo',color='r')
  #ax2 = fig.add_subplot(212)
  #ax2.plot(ccf,label='drive',color='k')
  #plt.savefig('test_ccf_fig'+np.str(i)+'.pdf')
  
 
  #plt.scatter(lc1[id1,0],lc1[id1,1],color='k')
  #plt.scatter(lc2[id2,0],lc2[id2,1],color='r')
  #plt.show()
 return(np.array(lag_ccf),ccf_save,np.array(lagpeak),np.array(ccfpeak),np.array(lagcent))
 
# 
#
# return(




#t  = np.arange(240)
#nt = t.size
#x =  np.zeros(nt)
#x[10:13] = 1.0
#lag = np.zeros(nt)
#lag[12:15] = 0.5
#
#lc1 = np.array([t,x]).T
#lc2 = np.array([t,lag]).T
#nsim = 500
#lag_ccf,ccf_save,lagpeak,ccfpeak = ccf_frrss(lc1,lc2,dt=0.5,fraction_rss=0.8,nsim =nsim)
#
#
#
#fig = plt.figure()
#ax1 = fig.add_subplot(411)
#ax1.plot(t,x)
#ax2 = fig.add_subplot(412)
#ax2.plot(t,lag)
#ax3 = fig.add_subplot(413)
#ax3.hist(lagpeak,bins=50)
#ax4 = fig.add_subplot(414)
##
##ccf = np.correlate((x-x.mean())/x.std(),(lag-lag.mean())/lag.std(),mode='full')
##ax4.plot(lag_ccf,ccf)
#[ax4.plot(lag_ccf,ccf_save[i],color='k',label=None) for i in range(nsim)]
#plt.show()
#







#sample code to test
#ilag = 5
#sigma = 0.3
#
##generate synthetic data
#drive = mlc.mylcgen(datfile='',p0=1.0,f0=0.1,a=-2,b=-2,tlo=0,thi=100,dt=1.0,ploton=0,iseed=-1,meannorm = -1., sdnorm = -1.0)
#
#t = drive[:,0]
#x = drive[:,1]
#xsd = np.std(x)
#nx = np.shape(x)[0] 
#x = (x - np.mean(x))/xsd
#xsd = np.std(x)
#sigx = np.ones(nx)*xsd*sigma
#x = x + np.random.randn(nx)*sigx
#
#lc1 = np.array([t,x,sigx]).T
#
##define convolution kernel
#conv = np.zeros(nx)
#conv[ilag-1:ilag+1] = 1
#
#
#
#echo  = np.convolve(x, conv, mode='same')
#echo2 = np.convolve(x, conv, mode='full')[:nx]
#
#echo2 = (echo2 - np.mean(echo2))/np.std(echo2) 
#esd = np.std(echo2)
#ne2=np.shape(echo2)[0]
#sige = np.zeros(ne2)+esd*sigma
#echo2 = echo2 + np.random.randn(ne2)*esd*sigma
#lc2 = np.array([t,echo2,sige]).T











'''
    A simple implementation of the discrete correlation function (DCF)
    Author: Damien Robertson - robertsondamien@gmail.com
    Usage:
      $ python dcf.py -h for help and basic instruction
'''

from __future__ import print_function, division

print(__doc__)

try:
    import numpy as np
except ImportError:
    print("Numpy not installed, try - pip install numpy")
    import sys
    sys.exit()

#
#   Subroutines
#

def tsdtrnd(ts, vrbs, plyft):

    '''
        Subroutine - tsdtrnd
          Time series detrend using the user chosen polynomial order. Subroutine
          fits a ploynomial to the time series data and subtracts.
        Requires scipy.optimize (scipy) to be installed.
    '''

    if plyft == 0:

        ts_mean = np.mean(ts[:,1])
        ts[:,1] = ts[:,1] - ts_mean
        if vrbs:
            print("Mean subtraction: %.4e" %ts_mean)

    elif plyft == 1:

        try:
            from scipy.optimize import curve_fit
        except ImportError:
            print("Scipy not installed, try - pip install scipy")
            import sys
            sys.exit()

        lnfnc = lambda x, a, b: a*x + b
        p0, c0 = curve_fit(lnfnc, ts[:,0], ts[:,1], sigma=ts[:,2])
        ts[:,1] = ts[:,1] - lnfnc(ts[:,0], p0[0], p0[1])

        if vrbs:
            print("Linear De-trend Coefficients [a*x + b]")
            print("a:", p0[0])
            print("b:", p0[1])

    else:

        try:
            from scipy.optimize import curve_fit
        except ImportError:
            print("Scipy not installed, try - pip install scipy")
            import sys
            sys.exit()

        lnfnc = lambda x, a, b, c: a*x**2.0 + b*x + c
        p0, c0 = curve_fit(lnfnc, ts[:,0], ts[:,1], sigma=ts[:,2])
        ts[:,1] = ts[:,1] - lnfnc(ts[:,0], p0[0], p0[1], p0[2])

        if vrbs:
            print("Quadratic De-trend Coefficients [a*x**2 + b*x + c]")
            print("a:", p0[0])
            print("b:", p0[1])
            print("c:", p0[2])

    return ts

def set_unitytime(ts1, ts2):

    '''
        Subroutine - set_unitytime
          Simply shifts both time series so that one starts at zero.
    '''

    unitytime = min(np.min(ts1[:,0]), np.min(ts2[:,0]))
    ts1[:,0] = ts1[:,0] - unitytime
    ts2[:,0] = ts2[:,0] - unitytime

    return ts1, ts2

def chck_tserr(ts):

    '''
        Subroutine - chck_tserr
          Makes sure user has entered a properly formatted ts file.
          Checks to see if input time series has a measurement error column - third
          column of input file.
    '''

    assert ((ts.shape[1] == 2) or (ts.shape[1] == 3)), "TS SHAPE ERROR"

    if ts.shape[1] == 2:
        ts_fill = np.zeros((ts.shape[0], 3))
        ts_fill[:,0:2] = ts[:,0:2]

        return ts_fill

    else:

        return ts

def get_timeseries(infile1, infile2, vrbs, plyft):

    '''
        Subroutine - get_timeseries
          Takes the user specified filenames and runs tsdtrnd and set_unitytime.
          Returns the prepared time series for DCF.
    '''

    ts1_in = np.loadtxt(infile1, comments='#', delimiter=',')
    ts2_in = np.loadtxt(infile2, comments='#', delimiter=',')

    ts1 = chck_tserr(ts1_in)
    ts2 = chck_tserr(ts2_in)

    ts1, ts2 = set_unitytime(ts1, ts2)
    ts1 = tsdtrnd(ts1, vrbs, plyft)
    ts2 = tsdtrnd(ts2, vrbs, plyft)

    return ts1, ts2

def sdcf(ts1, ts2, t, dt):

    '''
        Subroutine - sdcf
          DCF algorithm with slot weighting
    '''

    dcf = np.zeros(t.shape[0])
    dcferr = np.zeros(t.shape[0])
    n = np.zeros(t.shape[0])

    dst = np.empty((ts1.shape[0], ts2.shape[0]))
    for i in range(ts1.shape[0]):
        for j in range(ts2.shape[0]):
            dst[i,j] = ts2[j,0] - ts1[i,0]

    for k in range(t.shape[0]):
        tlo = t[k] - dt/2.0
        thi = t[k] + dt/2.0
        ts1idx, ts2idx = np.where((dst < thi) & (dst > tlo))

        mts2 = np.mean(ts2[ts2idx,1])
        mts1 = np.mean(ts1[ts1idx,1])
        n[k] = ts1idx.shape[0]

        dcfdnm = np.sqrt((np.var(ts1[ts1idx,1]) - np.mean(ts1[ts1idx,2])**2) \
                         * (np.var(ts2[ts2idx,1]) - np.mean(ts2[ts2idx,2])**2))

        dcfs = (ts2[ts2idx,1] - mts2) * (ts1[ts1idx,1] - mts1) / dcfdnm

        dcf[k] = np.sum(dcfs) / float(n[k])
        dcferr[k] = np.sqrt(np.sum((dcfs - dcf[k])**2)) / float(n[k] - 1)

    return dcf, dcferr

def gdcf(ts1, ts2, t, dt):

    '''
        Subroutine - gdcf
          DCF algorithm with gaussian weighting
    '''

    h = dt/4.0
    gkrn = lambda x: np.exp(-1.0 * np.abs(x)**2 / (2.0 * h**2)) \
           / np.sqrt(2.0 * np.pi * h)
    cntrbt = gkrn(3.290527*h)

    dcf = np.zeros(t.shape[0])
    dcferr = np.zeros(t.shape[0])
    n = np.zeros(t.shape[0])

    dst = np.empty((ts1.shape[0], ts2.shape[0]))
    for i in range(ts1.shape[0]):
        for j in range(ts2.shape[0]):
            dst[i,j] = ts2[j,0] - ts1[i,0]

    for k in range(t.shape[0]):
        gdst = gkrn(dst - t[k])
        ts1idx, ts2idx = np.where(gdst >= cntrbt)

        mts2 = np.mean(ts2[ts2idx,1])
        mts1 = np.mean(ts1[ts1idx,1])
        n[k] = ts1idx.shape[0]

        dcfdnm = np.sqrt((np.var(ts1[ts1idx,1]) - np.mean(ts1[ts1idx,2])**2) \
                         * (np.var(ts2[ts2idx,1]) - np.mean(ts2[ts2idx,2])**2))

        dcfs = (ts2[ts2idx,1] - mts2) * (ts1[ts1idx,1] - mts1) / dcfdnm
        dcf[k] = np.sum(dcfs) / float(n[k])
        dcferr[k] = np.sqrt(np.sum((dcfs - dcf[k])**2)) / float(n[k] - 1)

    return dcf, dcferr

#
#    MAIN PROGRAM
#

import argparse

INPUT = argparse.ArgumentParser(description='DCF USER PARAMETERS')

#
#    USER PARAMETER INPUT
#      STANDARD PARAMETERS (REQUIRED):
#        time_series1.dat - path/filename
#        time_series2.dat - path/filename
#        lag_range_low    - float
#        lag_range_high   - float
#        lag_bin_width    - float
#

INPUT.add_argument('infile1', metavar='time_series1', type=open, nargs=1,
                   help='Time Series 1')

INPUT.add_argument('infile2', metavar='time_series2', type=open, nargs=1,
                   help='Time Series 2')

INPUT.add_argument('lgl', metavar='lag_range_low', type=float, nargs=1,
                   help='Lag range low')

INPUT.add_argument('lgh', metavar='lag_range_high', type=float, nargs=1,
                   help='Lag range high')

INPUT.add_argument('dt', metavar='lag_bin_width', type=float, nargs=1,
                   help='Width of lag bin, dt')

#
#   USER PARAMETER INPUT
#     OPTIONAL PARAMETERS:
#       weight   = 'slot' or 'gauss'
#       polyfit  = 0, 1, 2
#       plot     = True or False
#       verbose  = True or False
#       output   = True of False
#

INPUT.add_argument('-w', dest='weight', type=str, nargs=1,
                   default=['slot'], choices=['slot', 'gauss'],
                   required=False, help='Lag bin weighting scheme')

INPUT.add_argument('-p', dest='polyfit', type=int, nargs=1,
                   default=[0], choices=[0, 1, 2],
                   required=False, help='Polynomial fit subtraction')

INPUT.add_argument('-np', '--no-plot', dest='noplot', action='store_false',
                   help='Do not produce plot')

INPUT.add_argument('-o', '--output', dest='output', action='store_true',
                   help='Write output file')

INPUT.add_argument('-v', '--verbose', dest='verbose', action='store_true',
                   help='Show all steps')

OPTS = INPUT.parse_args()

#
#   USER PARAMETER CHECK AND READOUT
#     This section will fail if:
#       Parameters 'lag_range_low' and 'lag_range_high' are not symmetric
#           about zero, ie: |lag_range_low| == |lag_range_high|
#       Parameter 'lag_range_low' is greater than 'lag_range_high'.
#
#   **PITFALL**
#     There is no check to make sure the user enters a sensible
#     lag bin width. See README for more details.
#

assert OPTS.lgl[0] < OPTS.lgh[0], "INPUT ERROR - LAG RANGE"

if OPTS.verbose:

    print("\nPYTHON SCRIPT: dcf3")
    print()
    print("INPUT TIMESERIES 1:", OPTS.infile1[0].name)
    print("INPUT TIMESERIES 2:", OPTS.infile2[0].name)
    print("LAG RANGE PROBED  :", OPTS.lgl[0], " : ", OPTS.lgh[0])
    print("LAG BIN WIDTH     :", OPTS.dt[0])

#
#   TIME SERIES PREPARATION
#     This section subtracts a n'th order polynomial from both input time
#     series prior to the DCF. The user may choose:
#       0'th order polynomial - subtracting the mean or zeroing the data.
#       1'st order polynomial - subtracting a linear fit
#       2'nd order polynomial - subtracting a quadratic fit
#     The default setting is subtracting a 0'th order polynomial (the mean).
#     This simply zeros the data and doesn't change any intrinsic qualities.
#
#   **PITFALL**
#     Just because you can subtract a n'th order polynomial doesn't mean you
#     should. The program doesn't monitor or tell you a subtraction is
#     harmful or unnecessary. 
#
#   **PITFALL 2**
#     If you have subtracted your own fits from the time series, leave the
#     default setting, 0, as is. It won't change your data.
#

if OPTS.verbose:

    print("\nTime series preparation")

TS1, TS2 = get_timeseries(OPTS.infile1[0], OPTS.infile2[0], OPTS.verbose, \
                          OPTS.polyfit[0])

#
#   DCF
#     This section earns the paycheck for the entire program - runs the DCF
#     algorithm. The user main choose the rectangular 'slot' weighting or
#     the gaussian 'gauss' weighting. See README for details on pair weighting.
#
#   The regular weighting scheme is 'slot' and also default. The 'gauss'
#   weighting assigns higher importance to data found at the centre of
#   the lag bin.
#

DT = OPTS.dt[0]
N = np.around((OPTS.lgh[0] - OPTS.lgl[0]) / float(DT))
T = np.linspace(OPTS.lgl[0]+(DT/2.0), OPTS.lgh[0]-(DT/2.0), N)

if OPTS.weight[0] == 'slot':

    if OPTS.verbose:

        print("\nDCF INITIATED USING SLOT WEIGHTING")

    DCF, DCFERR = sdcf(TS1, TS2, T, DT)
else:

    if OPTS.verbose:

        print("\nDCF INITIATED USING GAUSSIAN WEIGHTING")

    DCF, DCFERR = gdcf(TS1, TS2, T, DT)

if OPTS.verbose:

    print("DCF COMPLETE")

#
#   CHECK IF OUTPUT WRITE IS TRUE
#

if OPTS.output:

    print("Writing DCF output file to: dcf_output.csv")
    np.savetxt('dcf_output.csv', np.transpose((T, DCF, DCFERR)), fmt="%.6f", \
                header="LAG,DCF,DCF_ERROR", delimiter=',')

#
#   PLOT RESULTS
#     If the user wishes to suppress the plot
#     one should use the -np or --no-plot flag on the command line.
#
#   Requires python module matplotlib.
#

if OPTS.noplot:

    try:
        import matplotlib.pyplot as plt
    except ImportError:
        print("Matplotlib not installed, try - pip install matplotlib")
        import sys
        sys.exit()

    plt.figure(0)
    plt.errorbar(T, DCF, DCFERR, color='k', ls='-', capsize=0)
    plt.xlabel("Lag")
    plt.ylabel("Correlation Coefficient")
    plt.xlim(OPTS.lgl[0], OPTS.lgh[0])
plt.show()


import numpy as np
import astropy.convolution as apc
import vortexa_mod_randomwalk as mf18
import mylcgen as mlc
import myresample as mrs
import itertools
import matplotlib.gridspec as gridspec
import matplotlib.pylab as plt
import pandas as pd
import vortexa_convert_df2lc as c2l

#enter the raw time series data.
#interpolate using random walk model,
#convolve this with kernel,
#interpolate onto response data 
#and evaluate chi-squared 
def rw_convolve(drive,response,kernel,monte_carlo_iterations=1,summary_plot = '',
xgrid_in = [],
tgrid_in = [],
noise_in = []):

 #modify kernel to make compatible with astropy convolve
 laggrid = kernel[:,0]
 dtgrid = np.mean(laggrid[1:] - laggrid[:-1])
 ngrid = np.shape(laggrid)[0]
 id0 = np.where(laggrid >= 0)[0][0]
 n_neg = ngrid - id0 - 1
 n_pad = n_neg - id0
 pad = np.zeros(n_pad)
 kernel_input = np.append(pad,kernel[:,1])
 nki = np.shape(kernel_input)[0]
 
 
 #compute the overlapping time boundaries for variance and interpolation calculation
 tol_min = np.max([np.min(drive[:,0]),np.min(response[:,0])])
 tol_max = np.min([np.max(drive[:,0]),np.max(response[:,0])])
 idxinc  = np.where((response[:,0] > tol_min) & (response[:,0] < tol_max))
 ninc    = np.shape(idxinc)[0]
 
 
 #interpolate driver onto time grid using random walk
 if (tgrid_in == []):
  tlo,thi = np.min(drive[:,0]),np.max(drive[:,0])
  tgrid = np.arange(tlo,thi+dtgrid,dtgrid)
 else:
  tgrid = tgrid_in
  tlo = tgrid[0]
  thi = tgrid[-1]
  print('be careful when manually supplying tgrid \
  must have same spacing as lag grid',tgrid[1]-tgrid[0],dtgrid)
 
 
 ndrive = np.shape(drive[:,0])[0]
 fhi = 1./dtgrid
 flo = 0.5/(thi-tlo)
 nf = (fhi - flo)/flo
 nf= min(nf,1000)
 fhi = nf*flo
 
 print('fitting driver. Number of frequencies',nf,flo,fhi)
 if (xgrid_in == []):
  
  
  x = mf18.fitrw([drive[:,0]],[drive[:,1]],[np.ones(ndrive)*0.1*np.std(drive[:,1])],floin=flo,
  fhiin=fhi,plot_tit='',dtresin=tgrid,nits = monte_carlo_iterations,extra_f=[])
  s_model = x[4]
  s_noise = x[5]
  all_models = x[12]
  response_models = np.array( [apc.convolve(all_models[:,i],kernel_input,boundary = 'extend') for i in range(monte_carlo_iterations)] ).T
  response_mean = np.mean(response_models,axis=1)
 else:
  s_model = xgrid_in
  s_noise = noise_in
  response_mean = apc.convolve(xgrid_in,kernel_input)
 print('done fitting driver',drive[0,0],drive[-1,0],ndrive)
 
 
 #interpolate model response onto data time grid
 rt_itp = np.interp(response[:,0],tgrid,response_mean)
 
 #compute the model accuracy using the raw inputted response time series
 mean_response_model = np.mean(rt_itp[idxinc])
 rms_response_model  = np.std(rt_itp[idxinc])
 mean_response_data  = np.mean(response[:,1])
 rms_response_data    = np.std(response[:,1])
 
 
 
 response_transform  = (response_mean - mean_response_model)*rms_response_data/rms_response_model + mean_response_data
 
 n_response = np.shape(response[:,0])[0]
 variance = np.sum((rt_itp[idxinc]-response[idxinc,1])**2)/ninc
 #print('model cisquared/N: ',cisq)
 
 
 
 
 
 
 
 #summary plot
 if (summary_plot != ''):
  import matplotlib.pylab as plt
  fig = plt.figure()
  ax1 = fig.add_subplot(311)
  ax1.plot(drive[:,0],drive[:,1],marker='o',ls='')
  ax1.plot(tgrid,s_model,color='b')
  #ax1.fill_between(tgrid,s_model-s_noise,s_model+s_noise,alpha=0.3,color='b')
  
  ax2 = fig.add_subplot(312)
  ax2.plot(response[:,0],response[:,1],color='r',marker='o',ls='')
  ax2.plot(tgrid,response_transform)
  #[ax2.plot(tgrid,response_models[:,i],color='r') for i in range(monte_carlo_iterations)]
  
  ax3 = fig.add_subplot(313)
  ax3.plot(kernel[:,0],kernel[:,1],color='r')
  
  ax1.set_xlim([tlo,thi])
  ax2.set_xlim([tlo,thi])
  if (summary_plot == 'show'):
   plt.show()
  else:
   plt.savefig(summary_plot)
  plt.clf()
 
 
 return(response_transform,s_model,variance,s_noise)
















#design wrapper for above function to insert into fixtures code
def rw_convolve_gridsearch(lc1,lc2,
kernel_model='gaussian',
pars=[np.arange(0.0,55.0,5.0),np.arange(1.0,5.0,1.0)],
laggrid = np.arange(0.0,50.0,0.2),
tgrid_in = []):

 #specify custom time grid
 if (tgrid_in == []):
  tlo = np.min(np.array([np.min(lc1[:,0]),np.min(lc2[:,0])]))
  thi = np.max(np.array([np.max(lc1[:,0]),np.max(lc2[:,0])]))
  dtgrid = laggrid[1] - laggrid[0]
  tgrid = np.arange(tlo,thi+dtgrid,dtgrid)
  lg = np.array(laggrid)
 else:
  tgrid = tgrid_in
  dtgrid = tgrid[1]-tgrid[0]
  lg = np.arange(laggrid[0],laggrid[-1]+dtgrid,dtgrid)
  
  
 npars = len(pars)
 pars_unique = list(itertools.product(*pars))
 npars_unique = len(pars_unique)
 var_list = []
 par_list = []
 model_list = [] 
 for ip in range(npars_unique):
  pnow = list(pars_unique[ip])
  if (kernel_model == 'gaussian'):
   centroid = pnow[0]
   width    = pnow[1]
   kern   = np.exp(-0.5*((lg - centroid)/width)**2)
   kernel = np.array([lg,kern]).T
  else:
   raise Exception('please enter a valid kernel function')
   
  #only need to calculate random walk model on first iteration (I think) 
  if (ip == 0):
   x = rw_convolve(lc1,lc2,kernel,monte_carlo_iterations=1,summary_plot = '',tgrid_in = tgrid)
  else:
   x = rw_convolve(lc1,lc2,kernel,monte_carlo_iterations=1,summary_plot = '',
   tgrid_in = tgrid,xgrid_in = s_model,noise_in = s_noise )
   
  response,s_model,variance,s_noise = x
   
  var_list.append( variance )
  par_list.append( pnow )
  model_list.append( response ) 

 var_list = np.array(var_list)
 par_list = np.array(par_list)
 #model_list = model_list
 
 
 
 
 idx_best = np.argmin(var_list)
 model_best = model_list[idx_best]
 var_best   = var_list[idx_best]
 par_best   = par_list[idx_best]




 return(var_best,par_best,model_best,var_list,par_list,model_list,s_model,s_noise) 










#make syntax correct for data frame input
def mass_convolve(meta_df,dateref=-1,forecast_period = 30,figure_title=''):

 
 nframes = len(meta_df)
 labels  = ['time series '+np.str(i) for i in range(nframes)]
 color   = ['k','r','b','cyan','purple','green']*nframes

 #identify earliest date from all the time axis and use this as the first time by default
 datemax_save = []
 if (dateref == -1):
  dates = pd.concat([meta_df[i].iloc[:,0] for i in range(nframes)])
  datemax_save.append([meta_df[i].iloc[-1,0] for i in range(nframes)])
  datemin = min(dates)
  datemax = max(dates) + pd.Timedelta(days=forecast_period)
  date_trunc = np.min(np.array(datemax_save))
  print ('Earliest dat in sample...',datemin)
 else:
  datemin = dateref[0]
  datemax = dateref[1] + pd.Timedelta(days=forecast_period)
  date_trunc = dateref[1]



 #convert data frames to light curve format
 meta_lc = []
 for i in range(nframes):
  x = c2l.convert_df_2_lightcurve(meta_df[i],reftime=datemin,noise = 0.01,normalise = 0)
  meta_lc.append(x)
 



 #fit the convolved rw model to all the light curves relative to the flows
 tlo = 0
 thi = (datemax - datemin).days + forecast_period
 dtgrid = 1.0
 timegrid = np.arange(tlo,thi+dtgrid,dtgrid)

 timegrid_dates = np.array([pd.Timestamp(datemin) + pd.Timedelta(days=tgnow) for tgnow in timegrid])     
 ntgrid = np.shape(timegrid)[0]
 meta_model = []
 laggrid = np.arange(0.0,50.0,0.2)
 for i in range(nframes):
  x = rw_convolve_gridsearch(meta_lc[i],meta_lc[0],
    kernel_model='gaussian',
    pars=[np.arange(0.0,55.0,5.0),np.arange(1.0,5.0,1.0)],
    laggrid = laggrid,
    tgrid_in = timegrid)
  var_best,par_best,model_best,var_list,par_list,model_list,s_model,s_noise = x
  noise = np.ones(ntgrid)*np.sqrt(var_best)
  a_s = [timegrid_dates,s_model,s_noise,model_best,noise,var_best,par_best]
  
  print('model means')
  print(np.mean(s_model),np.mean(model_best),np.mean(meta_lc[i][:,1]),np.mean(meta_df[i].values[:,1]))
  print('model std')
  print(np.std(s_model),np.std(model_best),np.std(meta_lc[i][:,1]),np.std(meta_df[i].values[:,1]))
  #input()
  

  meta_model.append(a_s)



 
 #produce figures for each fit
 if (figure_title != ''):
  ndown = nframes + 2
  nalong = 3
  fig = plt.figure()
  gs1 = gridspec.GridSpec(ndown,nalong)
  gs1.update(left=0.15,right=0.85,wspace=0.05,hspace=0.0,bottom=0.1,top=0.99)
  
  
  for i in range(nframes-1):
   ax1 = plt.subplot(gs1[i,:])
   #ax1.plot(meta_df[i].values[:,0],meta_df[i].values[:,1],marker='o',ls='',label=labels[i],color=color[i])
   ax1.plot(meta_df[0].values[:,0],meta_df[0].values[:,1],marker='o',ls='',label=labels[0],color=color[0])
   
   ax1.set_xlim([timegrid_dates[0],timegrid_dates[-1]])
   ax1.tick_params(axis='x',rotation=30)
   driver_time   = meta_model[i+1][0]
   driver_signal = meta_model[i+1][1]
   driver_noise  = meta_model[i+1][2]
   response_signal = meta_model[i+1][3]
   response_noise  = meta_model[i+1][4]
   if (i < nframes - 1):
    ax1.set_xticks([])
   
   ax1.plot(driver_time,response_signal,label='convolve model '+labels[i+1],color=color[i])
   
   #ax1.fill_between(driver_time,response_signal - response_noise, response_signal + response_noise,label='convolve model '+labels[i+1],color=color[i])
   
   #ax1.fill_between(driver_time,driver_signal - driver_noise, driver_signal + driver_noise,label='convolve model '+labels[i+1],color=color[i])
   print(driver_time)
   print(np.mean(driver_signal))
   print(np.mean(driver_noise))
   
   
   #if (i == 0):
   # for i2 in range(1,nframes):
     
   #  rs = meta_model[i2][3]
   #  rn = meta_model[i2][4]
   #  ax1.fill_between(driver_time,rs-rn,rs+rn,alpha=0.4) 
  
  
  
  #construct bar plot of variances for each driving model 
  ax1 = plt.subplot(gs1[ndown-1,0])
  labels = ['time series '+np.str(i) for i in range(nframes)]
  xpos = np.arange(nframes)
  vars = [meta_model[i][5] for i in range(nframes)]
  [ax1.barh([xpos[i]],[vars[i]],label=None,color=color[i]) for i in range(nframes)]
  ax1.set_title('model variances')
  ax1.set_yticks(xpos)
  ax1.set_yticklabels(labels)
  ax1.tick_params(axis='y',rotation=0)
  

  
  #construct a plot of each response function model weighted by variances
  ax1 = plt.subplot(gs1[ndown-1,2])
  ax1.set_xlabel('lag')
  ax1.set_ylabel('response')
  for i in range(nframes):
   centroid,width = meta_model[i][6]
   resp = np.exp(-0.5*((laggrid - centroid)/width)**2)
   ax1.plot( laggrid, resp, label = labels[i],color=color[i] )
  ax1.legend()
  
 
  if (figure_title == 'show'):
   plt.show()
  else:
   plt.savefig(figure_title)


 return()



#
#
###test the lag_rwmod code
# 
#
#forecast = 180.0
#
#sddrive = 100.0
#meandrive = 0.0
#noise_drive = 0.8
#
#
#
#lagrange = [-100,100]
#sdecho = 1.0
#meanecho = 0.0
#noise_echo = 0.8 
#tlo = 0.0
#thi = 800.0
#dt  = 30.0
#gridres = 1.0
#lagres = 0.1
#nsim = 500
#
#
#
#
##impulse info
#lagcent = 50.0
#lagwide = 2.0
#lag = np.arange(0,lagrange[1]+lagres,lagres)
#response = np.exp(-0.5*((lag - lagcent)/lagwide)**2)
#
#
#
#
#
#
#
#
#
#datpre      = mlc.mylcgen(tlo=tlo-lagrange[1],thi=thi,dt=lagres,iseed=-1)#132423
#
#
#
#idlo  = np.where(datpre[:,0] > tlo)[0][0]
#npre     = np.shape(datpre[:,0])[0]
#
#
#datstd  = np.std(datpre[:,1])
#snow =  np.ones(npre)*noise_drive*datstd
#dat = mrs.myresample(dir='',fname=[''],dtave=dt,sampmin=20.,sampcode=3,datin=np.array((datpre[idlo:,0],datpre[idlo:,1],snow[idlo:])).T)
#ndat = np.shape(dat[:,0])[0]
#sig = dat[:,2]
#
#stdnow = np.std(dat[:,1])
#dat[:,1] = (dat[:,1] - np.mean(dat[:,1]))/stdnow*sddrive + meandrive
#dat[:,2] = dat[:,2]/stdnow*sddrive*noise_drive
##dat[:,1] = dat[:,1] + np.random.randn(ndat)*dat[:,2]
#
#
#
#x_drive = dat[:,1]
#errors_drive = dat[:,2]
#nx = np.shape(x_drive)[0]
#
#
#
#
#
#
#
#
#
##make a convolved version of the driver
#import myconvolve as  mc3
#import matplotlib.pylab as plt
#echo = mc3.mc3(datpre[:,0],datpre[:,1],lag,response)
#
#
##plt.plot(datpre[:,0],datpre[:,1])
##plt.plot(datpre[:,0],echo)
##plt.show()
#
#necho = np.shape(echo)[0]
#echostd = np.std(echo)
#errors_echo = np.ones(necho)*noise_echo*echostd
#dat_echo = mrs.myresample(dir='',fname=[''],dtave=dt,sampmin=2.,sampcode=3,datin=np.array((datpre[:,0],echo,errors_echo)).T)
#ndat = np.shape(dat_echo[:,0])[0]
#
#echosdnow = np.std(dat_echo[:,1])
#echomeannow = np.mean(dat_echo[:,1])
#
#dat_echo[:,1] = (dat_echo[:,1] - echomeannow)/echosdnow*sdecho + meanecho
#sig_echo = dat_echo[:,2]
#dat_echo[:,2] = sig_echo/echosdnow*sdecho*noise_echo
#dat_echo[:,1] = dat_echo[:,1] #+ np.random.randn(ndat)*sig_echo
#
#idlo_echo = np.where(dat_echo[:,0] > tlo)[0][0]
#
#
### fit modified random walk model
##nsnow = np.shape(snow)[0]
##d1 = np.zeros((nsnow,3))
##d1[:,:2] = datpre
##d1[:,2] = snow
##n_echo = np.shape(errors_echo)[0]
##d2 = np.zeros((n_echo,3))
##d2[:,0] = datpre[:,0]
##d2[:,1] = echo
##d2[:,2] = errors_echo
##
##
##id1lo = np.where((d1[:,0]>=tlo) & (d1[:,0]<=thi))[0]
##dat_new = 1.*dat
##dat_new[:,0] = dat_new[:,0] - 30.0
##
#
#
#
#
#
#
#denew = np.array(dat_echo[idlo_echo:,:])
##denew[:,0] = denew[:,0] + 50.0
#signals = [dat,denew]#[ dat, dat_new ]#[d1[id1lo,:],d2[id1lo,:]]
#titles  = ['arb','flows']
#
#
#kernel = np.array([lag,response]).T
#
#rw_convolve(dat[:,:2],denew[:,:2],kernel,monte_carlo_iterations=1,summary_plot = '')
#
#
#
#
#
##try iterating for several different reponse functions
##time test the code
#
#import time
##impulse info
#lagcent = np.arange(0,100,10)
#lagwide = np.arange(1,10,1)
#lag = np.arange(0,lagrange[1]+lagres,lagres)
#
#icount = 0
#
#for lc in lagcent:
# for lw in lagwide:
#  response = np.exp(-0.5*((lag - 1.*lc)/1.*lw)**2)
#  rin = np.array([lag,response]).T
#  
#  tlo = time.time()
#  if (icount == 0):
#   x = rw_convolve(dat[:,:2],denew[:,:2],rin,monte_carlo_iterations=1,summary_plot = '')
#   xgrid_in = x[1]
#  else:
#   x = rw_convolve(dat[:,:2],denew[:,:2],rin,monte_carlo_iterations=1,summary_plot = '',xgrid_in = xgrid_in)
#  thi = time.time()
#  print('simulation ',icount,' run time',thi-tlo,' centroid',lc,' width',lw,' cisq',x[2]) 
#  print()
#  icount = icount + 1
#lagcent = 50.0
#lagwide = 0.1
#
#
#
#






#take an input datasset with class variables and return nfold sets of nsub (<= Ndat - Ntest) subsamples of this data 
# for cross validation. Ntest will be extracted from the orriginal dataset to serve as the test comparison

import numpy as np
def my_folds(data,clas,nfolds,ntest,nsub):
 ndat,ndim = np.shape(data)
 
 #randomly select ntest samples to use as the benchmark accuracy test and remove these from the input
 #data prior to nfolds subselection
 idtest = np.random.choice(np.arange(ndat), size=ntest, replace=False, p=None)
 dattest   = data[idtest,:]
 clastest  = clas[idtest]
 datasub = np.delete(data,idtest,axis=0)
 classub = np.delete(clas,idtest)
 #select nfold new training data sets based on subsample of orriginal training data
 #for cross validation
 ndatnew = ndat - ntest
 datfold = []
 clasfold = []
 for i in range(nfolds):
  idtest = np.random.choice(np.arange(ndatnew), size=nsub, replace=True, p=None)
  dsnow = datasub[idtest,:]
  csub  = classub[idtest]
  datfold.append(dsnow)
  clasfold.append(csub)
 
 return(datfold,clasfold,dattest,clastest)


import numpy as np
import matplotlib.pylab as plt
import mytemp0 as mt0
import scipy.stats as ss

twopi = np.pi*2
fourpi = 2*twopi
deg2rad = np.pi/180
planck = 6.626307004e-34
c      = 2.99792458e8
boltz  = 1.38064852e-23
uw     = 1.e-10


def pytfb_sub(taus,rgrid,ygrid,zx,tv4,ti4,cosang,wav):
 tauhi  = taus[-1]
 taulo  = taus[0]
 ntaus  = np.shape(taus)[0]
 psis   = np.zeros(ntaus)
 ux = planck * c / ( uw * boltz * wav)


 nrad = np.shape(rgrid)[0] 
 #each point will have a time delay and a weighting append these to a list
 #for each point in the disc
 
 #loop of azimuths
 delsave = []
 wsave   = []
 if (diagnose == 1):
  azsave = []
  rsave = []
 

 for i in range(nrad-1):
  ynow = ygrid[i]
  r    = rgrid[i]
  zsuby=zx-ynow
  dd=zsuby*zsuby+r*r
  d=sqrt(dd)

  #quick way 
  canow     = cosang[i]
  tv4now    = tv4[i]
  ti4now    = tiv[i]*canow
  
  ttot4now   = tv4now + ti4now
  
  T2         = np.sqrt(ttot4now)
  T          = np.sqrt(T2)
  T3         = T*T2



  

  radlo = r
  radhi = rgrid[i+1]
  rhalf = (radlo + radhi)/2
  ylo   = ynow
  yhi   = ygrid[i+1]
  dy    = yhi - ylo
  drad  = radhi - radlo
  
  #now azimuth grid
  azwidth = drad/radlo
  daz = azwidth
  azgrid = np.arange(0.0, twopi,azwidth)
  naz = np.shape(azgrid)[0]
  nazsub1 = naz - 1
  azgrid_s = azgrid[1:] - azgrid[:-1]
  
  #print drad, radhi, azwidth,'sdfsd',radhi*azwidth
  #print radlo, radhi, azwidth, drad, naz,'sdfsf'
  
  raz   = np.random.uniform(radlo,radhi,nazsub1)   
  yaz  = ylo + (raz - radlo)/drad*dy#*np.sqrt(raz*raz + zx2)
  az   = np.random.uniform(low=azgrid[:-1],high=azgrid[1:],size=nazsub1)#np.random.uniform(low=0,high=1,size=nazsub1)*azgrid_s + azgrid[:-1]#np.random.uniform(azgrid[:-1],azgrid[1:],1)
  caz  = np.cos(az)
  tdel=( zsuby*ci - raz*caz + d ) # calculate time delay in days

  #calculate the weights
  x=ux/T        # record h c /(lamda K T)
  integrand = fourpi*x*x/(np.cosh(x)-1.)    # the xx/(cosh(x)-1) is part of dBdT !! the integrand of the transfer function£
  solid_angbit = np.abs(np.cos(np.arctan(dy/drad)))
  dsa        = rhalf*dr*daz * solid_angbit
  dTdL      = 1./T3 * canow / dd
  weight=integrand*dsa*dTdL
  
  wsave.append([weight]*nazsub1)
  delsave.append(tdl)
  if (diagnose == 1):
   azsave.append(az)
   rsave.append(raz)


 
 #introduce a lower threshold weight to abort the iterations (to save time)
 
 
 
 delsave = [item for sublist in delsave for item in sublist]
 wsave   = [item for sublist in wsave for item in sublist]
 if (diagnose == 1):
  rsave   = [item for sublist in rsave for item in sublist]
  azsave  = [item for sublist in azsave for item in sublist]



 delsave = np.array(delsave)
 wsave   = np.array(wsave)
 if (diagnose == 1):
  rsave   = np.array(rsave)
  azsave  = np.array(azsave)
 
 nds = np.shape(delsave)[0]
 
 
 
 #add smoothing function to approximate delta function
 #ntaus = np.shape(taus)[0]
 sigtau = 5*dtau/2
 sigtaulim = 3*sigtau
 for i in range(ntaus):
  taunow = taus[i]
  idxlo = taunow - sigtaulim
  idxhi = taunow + sigtaulim
  idnow = np.where((delsave > idxlo) & (delsave < idxhi))[0]
  
  a = (taunow - delsave[idnow])/sigtau
  a2 = -a*a/2
  ea2 = np.exp(a2)
  #ea2sum = np.sum(ea2) dont seem to need ea2sum but put back in later if necessary
  ea2out= np.sum(wsave[idnow]*ea2)
  psis[i] = ea2out#/ea2sum
  

 
 psis = np.nan_to_num(psis,0)  

 if (norm == 1):
  pt   = (psis - np.min(psis))
  psis = pt/np.max(pt)
 

 
 if (diagnose == 1):
   return(psis,delsave,wsave,rsave,azsave)
 else:
  return(psis)
#






#outputs are psis[ntau] to input set of time grid[ntau]
#make code to test and plot make nice plots
# 
#import timeit 
#
#wavref = 4000.
#embhref   = 1.e8
#emdotref  = 1.0
#degref    = 0.0
#
#t0v   = -1.6e4
#t0i   = -1.2e4
#taus   = np.arange(0.0,30.,0.1)
#ntau = np.shape(taus)[0]
#
#
#
#
#
#
#color=['k','r','b','cyan','purple','orange','gree','skyplue','magenta']
#
#deg = [0.0,20.0,40.0,60.0,80.0]
#fig = plt.figure(111)
#ax1 = fig.add_subplot(111)
#idx = 0
#for dnow in deg:
# psis   = pytfb_sub(taus,embhref,emdotref,wavref, dnow,t0vin = t0v, t0iin = t0i, norm = 1,quick = 1,oldsmooth=0,newsmooth=1,diagnose = 0)
# ax1.plot(taus,psis,color=color[idx],label='deg = '+np.str(dnow))
# #calculate the mean
# mean = np.sum(taus*psis)/np.sum(psis)
# ax1.plot([mean,mean],[0.0,1.0],color=color[idx],label=None)
# idx = idx + 1
#ax1.set_xlabel('delay (days)')
#ax1.set_ylabel('response (arbitrary units)')
#plt.savefig('plot_response_inc.pdf')
#











#other diagnostic tests testing smoothing function 



#start = timeit.default_timer()
#psis   = pytfb_sub(taus,embh,emdot,wavang, deginc,t0vin = t0v, t0iin = t0i, norm = 1,quick = 1)
#
#ds = psis[1]
#ws = psis[-1]
#
#ida = np.argsort(ds)
#ds = ds[ida]
#ws = ws[ida]
#nws = len(ws)
#
##weights = np.zeros((ntau,nws))
#
##tauds = np.zeros((ntau,nws))
##tauds[:,:] = taus
##tauds[:,:] = taus - ds
#
##dssub = ds - taus
##weights = np.exp(ds - taus)
#
#
#psis = psis[0]
#stop = timeit.default_timer()
#
#
#
#print 'run time', stop-start
# 
#fig = plt.figure()
#ax1 = fig.add_subplot(111)

#ax1.plot(ds,ws/ws.max())


#start = timeit.default_timer()
#psis   = pytfb_sub(taus,embh,emdot,wavang, deginc,t0vin = t0v, t0iin = t0i, norm = 1,quick = 0)
#
#ds = psis[1]
#ws = psis[-1]
#
#ida = np.argsort(ds)
#ds = ds[ida]
#ws = ws[ida]
#nws = len(ws)
#
##weights = np.zeros((ntau,nws))
#
##tauds = np.zeros((ntau,nws))
##tauds[:,:] = taus
##tauds[:,:] = taus - ds
#
##dssub = ds - taus
##weights = np.exp(ds - taus)
#
#
#psis = psis[0]
#stop = timeit.default_timer()
#print 'run time', stop-start
#
#fig = plt.figure()
#ax1 = fig.add_subplot(111)
#ax1.plot(taus,psis,label='quick off old smooth')
#ax1.plot(ds,ws/ws.max(),label='quick off no smooth')
#










#experiment 2 
#
#psis   = pytfb_sub(taus,embh,emdot,wavang, deginc,t0vin = t0v, t0iin = t0i, norm = 1,quick = 0)
#ds = psis[1]
#ws = psis[-1]
#ida = np.argsort(ds)
#ds = ds[ida]
#ws = ws[ida]
#nws = len(ws)
#
#datnew = ss.binned_statistic(ds,ws,statistic='mean',bins=taus)
#a = datnew[0]
#a = np.insert(a,0,0)
#ax1.plot(taus,a)
#
#plt.savefig('pytfbplottest_quick.pdf')

#scipy.stats.binned_statistic(x, values, statistic='mean', bins=10, range=None)
#try making histogram of points and sampling mean from each bin



















#experiment 3 
#
#psis   = pytfb_sub(taus,embh,emdot,wavang, deginc,t0vin = t0v, t0iin = t0i, norm = 1,quick = 0)
#ds = psis[1]
#ws = psis[-1]
#ida = np.argsort(ds)
#ds = np.array(ds[ida])
#ws = np.array(ws[ida])
#nws = len(ws)
#dtau = taus[1] - taus[0]
#sigtau = 5.*dtau
#ntaus = np.shape(taus)[0]
#psinew = np.zeros(ntaus)
#
##taunew = np.insert(np.arange(-10.0
#for i in range(ntaus):
# taunow = taus[i]
# a = (taunow - ds)/sigtau
# a2 = -a*a/2
# ea2 = np.exp(a2)
# ea2sum = np.sum(ea2)
# ea2sum = np.sum(ws*ea2/ea2sum)
# psinew[i] = ea2sum
# 
#ax1.plot(taus,psinew/psinew.max(),label='quick off new smooth')
#ax1.legend()
#plt.savefig('pytfbplottest_quick.pdf')
#
#scipy.stats.binned_statistic(x, values, statistic='mean', bins=10, range=None)
#try making histogram of points and sampling mean from each bin






##experiment 3
#fig = plt.figure()
#ax1 = fig.add_subplot(111)
#
#print 'experiment 1 old smoothing quick = 1'
#start = timeit.default_timer()
#psis   = pytfb_sub(taus,embh,emdot,wavang, deginc,t0vin = t0v, t0iin = t0i, norm = 1,quick = 1,oldsmooth=1,newsmooth=0,diagnose = 1)
#stop = timeit.default_timer()
#print 'run time', stop-start
#ax1.plot(taus,psis[0]/psis[0].max(),label='experiment 1 old smoothing quick = 1')
#
#print 'experiment 2 no smoothing quick = 1'
#start = timeit.default_timer()
#psis   = pytfb_sub(taus,embh,emdot,wavang, deginc,t0vin = t0v, t0iin = t0i, norm = 1,quick = 1,oldsmooth=0,newsmooth=0, diagnose = 1)
#stop = timeit.default_timer()
#print 'run time', stop-start
#ax1.plot(psis[1],psis[2]/psis[2].max(),label='experiment 2 no smoothing quick = 1')
#
#
#
#print 'experiment 3 new smoothing quick = 1'
#start = timeit.default_timer()
#psis   = pytfb_sub(taus,embh,emdot,wavang, deginc,t0vin = t0v, t0iin = t0i, norm = 1,quick = 1,oldsmooth=0,newsmooth=1,diagnose = 0)
#stop = timeit.default_timer()
#print 'run time', stop-start
#ax1.plot(taus,psis/psis.max(),label='experiment 3 new smoothing quick = 1')
#
#ax1.legend()
#plt.savefig('pytfbplottest_quick.pdf')
#
#
#
#
###plot the azimuth and radius grid to identify the source of hte hump
#rsave = psis[3]
#azsave = psis[4]
#fig = plt.figure()
#ax1 = fig.add_subplot(311)
#ax1.hist(rsave,bins = 200)
#ax1 = fig.add_subplot(312)
#ax1.hist(azsave,bins = 200)
#ax1 = fig.add_subplot(313)
#ax1.hist(psis[1],bins = 200)
##plt.show()
#plt.savefig('pytfbplottest_hist.pdf')
#
#
##fig = plt.figure()
##ax1 = fig.add_subplot(111)
##ax1.scatter(rsave*np.cos(azsave),rsave*np.sin(azsave))
##plt.savefig('pytfbplottest_scatter.pdf')
#















import numpy as np
import os

#str_rep_in is a list [1:ncol][1:nrow] if you want to replace a column by strings, set str_rep_in[icol]='' if dnot want to alter
#code to make a latex table
#aw shift by xxcm
def mytextable(datin=[],filedatin='',opfile='mytable',caption='',colheadin='',ndpin=[],datsd=[],colunits='',str_rep_in=[],sideways=0,aw='0cm',tabref='',verbon = 1):

 
 

 if ((datin ==[]) & (filedatin == '')):
  raise Exception('mytextable.py: Both filedat ad dat arguments cannot be empty!')
 elif (datin == []):
  dat = np.loadtxt(fiiledatin)
 elif (filedatin==''):
  dat = 1.*datin
 else:
  raise Exception('mytextable.py: Something else is wrong!')
 
 
 
 
 ncol = np.shape(dat[0,:])[0]
 nrow = np.shape(dat[:,0])[0]
 
 
 
 if (str_rep_in == []):
  str_rep = ['']*ncol
 else:
  str_rep = str_rep_in
 
 if (datsd == []):
  datsdtab = np.ones((nrow,ncol))*-1.
 else:
  datsdtab = datsd

 
 if (ndpin ==[]):
  ndp = [2]*ncol
 else:
  ndp = list(ndpin)
 
 print ndp
 
 target = open(opfile+'.tex', 'w')
 
 cs = ''
 cs = [cs + 'c ' for i in np.arange(ncol)] 
 cs = ''.join(cs)+'}'
 cs = '{'+cs
 
 ch = ''
 if (colheadin == ''):
  ch = [ch + np.str(i) +' & ' for i in np.arange(ncol)]
 else:
  ch = [ch + i +' & ' for i in colheadin]
 ch = ''.join(ch)[:-2] +' \\\\ \n'

 cu = ''
 if (colunits == ''):
  cu = ''
 else:
  cu = [cu + i +' & ' for i in colunits]
 cu = ''.join(cu)[:-2] +' \\\\ \n'



 
 #write document ifo
 target.write('\\documentclass[10pt]{article}  \n')
 target.write('\\newcommand{\\HRule}{\\rule{\\linewidth}{0.5mm}} \n')
 target.write('\\parindent 0pt \n')
 target.write('\\parskip 10pt \n')
 target.write('\\usepackage{anysize} \n')
 target.write('\\usepackage{graphicx} \n')
 target.write('\\usepackage{rotating} \n')
 target.write('\\usepackage{epsfig} \n')
 target.write('\\usepackage{float} \n')
 target.write('\\usepackage{natbib} \n')
 target.write('\\usepackage{setspace} \n')
 target.write('\\usepackage{changepage} \n')
 target.write('\\marginsize{3.5cm}{3.5cm}{1cm}{1cm} \n')
 target.write('\\onehalfspacing \n')
 target.write('\\usepackage{caption} \n')
 target.write('\\usepackage{subcaption} \n')
 target.write('\\usepackage{amsmath} \n')
 target.write('\\begin{document} \n')
 
 
 #write preamble including table caption
 if (sideways == 1):
  target.write('\\begin{sidewaystable} \n')
 else:
  target.write('\\begin{table} \n')
  
 target.write('\\begin{adjustwidth}{'+aw+'}{} \n')
 target.write('\\caption{'+caption+'} \n')
 target.write('\\centering \n')
 target.write('\\begin{tabular}'+cs+' \n')
 target.write('\\hline\\hline \n')
 target.write(ch)
 if (colunits != ''):
  target.write(cu)
 target.write('\\hline \n')
 
 #now input table data
 #row by row
 
 for i in range(nrow):
  rownow = ''
  for i2 in range(ncol):
   
   
   if (str_rep[i2] == ''):
    
    if (datsdtab[i,i2] == -1):
     if (ndp[i2] != 0):
      rownow = rownow +  np.str(np.round(dat[i,i2],ndp[i2])) +' & '
     else:
      rownow = rownow +  np.str(np.int(dat[i,i2])) +' & '
    else:
     if (ndp[i2] != 0):
      rownow = rownow +  np.str(np.round(dat[i,i2],ndp[i2])) +' $\\pm$ '+ np.str(np.round(datsdtab[i,i2],ndp[i2])) + ' & '
     else:
      rownow = rownow +  np.str(np.int(dat[i,i2])) +' $\\pm$ '+ np.str(np.int(datsdtab[i,i2])) + ' & '
    
   else:
    if (verbon == 1):
     rownow = rownow + '\\verb|'+str_rep[i2][i] + '| & ' 
    else:
     rownow = rownow + str_rep[i2][i] + ' & ' 
  
  #rownow = [rownow + np.str(np.round(x,ndp[0])) +' & ' for x in dat[0,:]]
  #rownow = ''.join(rownow)[:-2] + '\\\\ \n'
  rownow = rownow[:-2] + '\\\\ \n'
  target.write(rownow)
 
 target.write('\\hline \n')
 target.write('\\end{tabular} \n')
 if (tabref != ''):
  target.write('\\label{'+tabref+'} \n')
 target.write('\\end{adjustwidth} \n')
 if (sideways==1):
  target.write('\\end{sidewaystable} \n')
 else:
  target.write('\\end{table} \n')
 
 target.write('\\end{document} \n')
 
 target.close()
 
 
 os.system('pdflatex '+opfile+'.tex '+opfile+'.pdf')
 #os.system('rm '+opfile+'.tex')
 os.system('rm '+opfile+'.aux')
 os.system('rm '+opfile+'.blg')
 os.system('rm '+opfile+'.log')
 os.system('rm '+opfile+'.synctex.gz')

 #print 'pdf2latex not available. I have just mde the tex file you will have to compile later'



#dat = np.ones((10,2))
#datsd = np.ones((10,2))
#datsd[:,-1] = -1
#datsd[3,0] = -1
#colheadin = ['col 1','col 2']
#units = ['','days']
#a = mytextable(datin = dat, datsd = datsd,colheadin = colheadin,colunits = units)
#
#


#combine 2textfiles into 1
import myreadlines 
import numpy as np

def mycomb2to1(fname,fnameop='',cutoff=[]):
 
 #
# with file(fname) as f:
#  fname = f.read().splitlines()
 
 nf  = len(fname)
 cutoff_in = []*nf
 #print fname,len(fname)
 
 if (len(cutoff) == 0):
  cutoff_in[:] = 0
 else:
  cutoff_in[:] = cutoff[:]
 
 
 dat=[]
 ncol = []
 for i in range(nf):
  try:
   temp = np.loadtxt(fname[i])
  except:
   temp = myreadlines.myreadlines(fname[i])
  
  print 'mycomb2to1.py:', fname[i],nf, temp.shape[0]
  #only read after the cutoff
  if (cutoff_in[i] < 0):
   nrolold = temp.shape[0] 
   nro =  abs(nrolold*cutoff_in[i])
  else:
   nco = cutoff_in[i]
  
  temp = temp[nro:,:]
   
  #make sure both files have same number of columns
  ncol.append(temp.shape[1])
  ncolmin = min(ncol)
  if (all(x==ncol[0] for x in ncol) ==0):
   print 'mycomb2to1 warning: different number of colums for the two arrays you are trying to combine. I will combine to the lowest number...'  
   print 'column lengths...', ncol
  dat.append(temp)
 
 datnew=[]
 for i in range(nf):
  datnew.append(dat[i][:,:ncolmin])
   
 datop = np.concatenate((datnew[:]),axis=0)
 
 print ''
 print 'mycomb2to1.py: after combining. New umber of points=',np.shape(datop) 
 if (fnameop != ''):
  np.savetxt(fnameop,datop)
 
 return(datop)


import scipy.optimize
import numpy as np
import myrandom as mr
import astropy.stats as ast
import matplotlib.pylab as plt
#jon trumps implementation of keiths intrinsic variability code
#
##function getrms, fluxes, errors
##
##;; Keith Horne's method to estimate intrinsic variability.
##;;
##;; V = sum( (xi-u)^2 gi^2 ) / sum(gi)
##;;   V = intrinsic variance
##;;   xi = flux measurement
##;;   u = mean flux
##;;   gi = 1 / (1+e^2/V)
##;;   e = flux error
##;;
##;; Iterate to solve for V.  Note that e should be a combination of
##;; the standard flux error plus the spectrophotometry error.
##
##  if stddev(fluxes) eq 0 or n_elements(fluxes) le 1 then return,[0,-1]
##
##  meanflux = median(fluxes)
##;;   errors = sqrt(errors^2 + (0.04*meanflux)^2)  ;add 0.04% specphoterror
##
##  varguess = (stddev(fluxes)^2 - median(errors)^2)>0.05
##
##  repeat begin
##     varold = varguess
##     gi = 1 / (1+errors^2/varold)
##     varnew = total( (fluxes-meanflux)^2 * gi^2 ) / total(gi)
##     varguess = varnew
##  endrep until abs(varold-varnew)/varnew le 1e-5 or varnew lt 1e-12
##
##  varerror = 2d / ( 2*total( (fluxes-meanflux)^2 / (varnew+errors^2)^3 ) $
##                            - total(1d/(varnew+errors^2)^2) )
##
##  return, [sqrt(varnew>0), sqrt(varerror>0)/sqrt(varnew>1e-4)]
##
##;  varerror2 = 2d * ( 2d*total((fluxes-meanflux)^2 * gi^3) $
##;                    / total((fluxes-meanflux)^2 * gi^2) * total(gi) - total(gi^2) )
##;
##;  return, [sqrt(varnew>0), sqrt(varnew>0)/sqrt(varerror2>1e-4)]
##
##end


#converted from Keiths fortran routine
#* =================================================
#	subroutine avgrmsx( n, dat, sig, good, avg, rms, sigavg, sigrms )
#* Maximum likelihood estimator of the mean and intrinsic rms
#* given independent samples with error bars
#* In:	n	i4 number of data
#*	dat(n)	r4 data
#*	sig(n)	r4 1-sigma error bars (<0 to skip)
#* Out:	good	r4 number of "good" data
#*	avg	r4 mean value
#*	sigavg	r4 1-sigma error bar
#*	rms	r4 excess rms
#*	sigrms	r4 1-sigma of excess rms
#* 2014 Aug Keith Horne @ St Andrews
#* 2014 Nov KDH @ StA - improve estimate of sigrms
#* 2016 Mar KDH @ StA - catch crash if no variance
#* 2016 Mar KDH @ StA - scale to <1/sig^2>=1. avoids vmin=0.


def myvarlc(fluxin,sigin):
 
 n = np.shape(fluxin)[0]

 #trap no data
 good = 0.
 avg = 0.
 rms = 0.
 sigavg = -1.
 sigrms = -1.
 if( n <= 0 ):
  print 'flux,sig arrays empty, no data',n
  return(good, avg, rms, sigavg, sigrms)

 
 #only use data with non zero error bar
 idinc = np.where(sigin > 0)[0]
 flux = fluxin[idinc]
 sig  = sigin[idinc]
 ng = np.shape(sig)[0]
 #trap no valid data
 if( ng <= 0 ):
  print '** ERROR in avgrmsx. n', n, ' ng', ng
  return(good, avg, rms, sigavg, sigrms)	
  
 #average positive error bars
 e1 = np.mean(sig)
 #KDH : V1 MAY VANISH IF E1 TOO SMALL
 v1 = e1 * e1
 
 
 #initial estimate
 x = e1/sig
 x2 = x*x
 sum1 = np.sum(flux*x2)
 sum2 = np.sum(x2)
 varguess = np.std(flux) 


 #optimal average and its variance
 avg = sum1 / sum2
 sigavg = e1 / np.sqrt( sum2 )
 v0 = e1 * ( sigavg / sum2 )

 #scale factor ( makes <1/sig^2>=1 )
 v1 = ng * e1 * ( e1 / sum2 )
 e1 = np.sqrt( v1 )
 v0 = v0 / v1
 
 
 #convergence threshold
 tiny = 2.e-5
 #lower limit for extra variance
 vmin = tiny * v0







#max-likelihood estimate of mean and extra varian
 nloop = 1000
 for loop in range(nloop):
  #stow for convergence test
  oldavg = avg
  oldrms = rms

  #data loop
  a = avg / e1
  
  d = flux/e1
  e = sig/e1
  #weight
  w = 1./(v0 + e*e)
  #"goodness" of the data 
  g = v0 * w
  x = g * (d - a)
  xx = x*x
  
  sum1 = np.sum(g*d)
  sumg = np.sum(g)
  sum  = np.sum(g*g)
  sum2 = np.sum(xx)
  sum3 = np.sum(g*xx)


  #"good" data points ( e.g. with sig < rms )
  good = sumg

  if( sumg < 0.0 ):
   print '** ERROR in AVGRMSX. NON-POSITIVE SUM(G)=', sumg
   print '** loop', loop, ' ndat', n, ' ngood', ng
   print '** sumg', sumg, ' sum1', sum1, ' sum', sum
   print '** tiny', tiny, ' v0', v0, ' vmin', vmin
   print '** dat(:)', flux[:]
   print '** sig(:)', sig[:]



  a = sum1 / sumg
  v0 = sum2 / sumg
  v0 = max( vmin, v0 )
  va = v0 / sumg
  #new avg and rms
  avg = a
  rms = np.sqrt( v0 )
  #error bars on avg and rms
  sigavg = np.sqrt( va )
  g = 2.0 * ( 2.0 * sum3 / sum2 * sumg - sum )
  
  
  
  sigrms = rms
  if( g > 0.0 ): 
   sigrms = rms / np.sqrt( g )

   
  #restore scaling
  avg = a * e1
  rms = rms * e1
  sigavg = sigavg * e1
  sigrms = sigrms * e1




  #KDH: CATCH SUM2=0 (NO VARIANCE)
  if( sum2 < 0.0):
   rms = 0.
   sigrms = -1.
   print 'problem CATCH SUM2=0 (NO VARIANCE)'
   return(good, avg, rms, sigavg, sigrms)

  #converge when test < 1
  if( loop > 1 ):
   safe = 0.9
   avg = oldavg * ( 1. - safe ) + safe * avg
   rms = oldrms * ( 1. - safe ) + safe * rms
   chiavg = ( avg - oldavg ) / sigavg
   chirms = ( rms - oldrms ) / sigrms
   test = max( abs( chiavg ), abs( chirms ) ) / tiny
   #report on last 5 iterations
   if( loop > nloop - 3 ):
	print 'Loop', loop, ' of', nloop, ' in AVGRMSX'
	print 'Ndat', n, ' Ngood', good, ' Neff', g
	print ' avg', avg, ' rms', rms
	print ' +/-', sigavg, ' +/-', sigrms
	print ' chiavg', chiavg, ' chirms', chirms, ' test', test
	if( test < 1. ):
	 print '** CONVERGED ** :))'

   #converged
   if( test < 1. ):
    return(good, avg, rms, sigavg, sigrms)

  #quit if v0 vanishes
  if( v0 <= 0. ): 
   return(good, avg, rms, sigavg, sigrms)

  #next loop


 #failed to converge
 print '** AVGRMSX FAILED TO CONVERGE :((((('
 return(good, avg, rms, sigavg, sigrms)






#python adaptation to jon trump iqrerror code
def iqrerror(flux,error):
 niter = 1000
 nv = np.shape(flux)[0]
 iqr = np.zeros(niter)

 for ee in range(niter):
  #for some reason jon trumps code doesn't use all the points
  #for the monte carlo resampling. Maybe to save time
  # i turned this off but you can put it back on if you want
  ###ind = long(randomu(seed,nv+1)*(nv+1))<nv
  
  flux1 = flux + np.random.randn(nv)*error
  #fluxsort = sort(flux1)

  x = np.percentile(flux1,[25,75])
  iqr[ee] = x[1]-x[0]
  #iqr[ee] = flux1[fluxsort[3*nv/4]] - flux1[fluxsort[nv/4]]
 return(np.std(0.74*iqr))









#linear fit
def func(x,a,b): 
 return(a+b*x)

#finction to fit linear trend to light curve and use above maximum likelihood estimator to 
#get the snr
#iterates linear fit over time using iq code to caclualte intrinsic errors
#iterativelt and refits linear line
#returns Keiths ML var and uncertainty
#snr in kates paper are ML/sigML from Keiths code i.e var[0]/var[1] (ML avgrms variance / uncertainty)
def linvarfit(time,flux,err,diag = 0,frac_intersect=0):
 
 iqr=0.0
 iter=0
 maxiter=10
 
 nv = np.shape(flux)[0]-1
 
 mederr = np.median(err)
 mederr2 = mederr*mederr
 iqr2 = iqr*iqr
 err2 = err*err
 toterr2 = err2+iqr2
 toterr  = np.sqrt(toterr2)
 x = np.median(np.abs(toterr2 - (err2 + iqr2))/flux) 
 c = ast.median_absolute_deviation(err2)
 c2 = c*c
 cc = c2/4/nv 
 
 #i dont think there is any need to iterate here 
 #but go ahead and see what happens
 while ((x < 1.e-4) and (iter <= maxiter)):
  
  
  #linear fit
  fit_coef=scipy.optimize.curve_fit(func,time,flux,sigma=toterr)
  npar = np.shape(fit_coef[1])[0]
  sig_coef = np.sqrt([[fit_coef[1][i,i]] for i in range(npar)])[:,0]
  fit_coef = fit_coef[0]  
  modelfit = fit_coef[0] + fit_coef[1]*time
  flux0 = flux - modelfit
  if (frac_intersect==1):
   idgood = np.where(np.abs(flux0) - toterr < 0)[0]
   ngood = np.shape(idgood)[0]
   npcgood = 1.*ngood/nv
  else:
   npcgood = 1.
  #make sure linear fit working properly
  if (diag==1):
   fig = plt.figure()
   ax1 = fig.add_subplot(111)
   ax1.errorbar(time,flux,toterr,ls='',capsize=2,label='toterr')
   ax1.errorbar(time,flux,err,ls='',capsize=1,label='errorbars')
   ax1.plot(time,modelfit,color='k',label='fit')
   ax1.set_title('iteration '+np.str(iter))
   plt.legend()
   plt.show()

  #ML variance estimator
  var_keith = myvarlc(flux0,err)
  var_trump = [var_keith[2],var_keith[2]/var_keith[4]]
  
  #IQR range of 25-75% of cumulative distribution,
  # with correction (x0.74) to equal sigma for a Gaussian
  x = np.percentile(flux0,[25,75])
  
  vsort = np.sort(flux0)
  iqr_obs = 0.74 * (x[1] - x[0])
  iqrerr_obs = iqrerror(flux0,err)
  iqr = np.sqrt(np.max(iqr_obs**2 - mederr2,0))
  iqr2 = iqr*iqr
 
  iqrerr = 1/iqr * np.sqrt( (iqr_obs*iqrerr_obs)**2 + cc)
  toterr2 = err2+iqr2
  toterr  = np.sqrt(toterr2)
  x = np.median(np.abs(toterr2 - (err2 + iqr2))/flux) 

#the below are another way of getting the snr still coded in c need to trnslate to 
#python one below half done dont need final one for routine

#AD: absolute average deviation (e.g. vanden Berk et al 2004)
#   aad = getaad(flux0,err)
#
  #print 'median error', np.median(err)
  print 'keith ml var and var/var_error',var_trump
  #print 'iqr and iqrerr', iqr,iqrerr
  
  iter = iter + 1

 if (frac_intersect==1):
  return(var_trump,npcgood)
 else:
  return(var_trump)













import numpy as np
import matplotlib.pylab as plt
import mytemp0 as mt0

twopi = np.pi*2
deg2rad = np.pi/180
planck = 6.626307004e-34
c      = 2.99792458e8
boltz  = 1.38064852e-23




def pytfb_sub(taus,embh,emdot,wavang, deginc, t0vin=-1, t0iin = -1, alpha_visc = -0.75, hxsch = 3.0, alpha_irad = -0.75, eta = 0.1, rlosch = 3.0, norm = 1, quick = 1, xstop = 15, udlnr = 0.01,thcent=1.0,thfwhm=0.2):
 tauhi  = taus[-1]
 ntaus  = np.shape(taus)[0]
 psis   = np.zeros(ntaus)
 
 #if you want a top hat response then its easy else do what you had before
 if (wavang < 0.0):
  idxinc = np.where((taus > thcent - thfwhm/2) & (taus < thcent + thfwhm/2))[0]
  psis[idxinc] = 1
 
 
 else:
 #either input desired reference temperature t0 at 1 light day, or calculate the
 #value appropriate for black body accretion disc given m and mdot 
  if (t0vin < 0):
   t0v   = mt0.tv0(embh,emdot)
  else:
   t0v   = t0vin
  
  if (t0iin < 0):
   t0i   = mt0.ti0(embh,emdot, eta = eta)
  else:
   t0i   = t0iin 
   
 
  #rlosch = 3.0
  #wavang = 4000.
  #hxsch  = 3.0
  
  
  nrad   = 10000
  
  #need to calculate 1/T^3 (r/r0)^alpha_irad x^2/wav^2 / (cosh(x) - 1) delta (tau - tau(r,theta)) dtau
  
  
  t0v2   = t0v*t0v
  t0v4   = t0v2*t0v2
  
  t0i2   = t0i*t0i
  t0i4   = t0i2*t0i2
  
  rsch   = 1.15821e-10*embh #scwarzchild radius in light days
  hx     = hxsch*rsch
  hx2    = hx*hx
  cosi   = np.cos(deginc*deg2rad)
  sini   = np.sin(deginc*deg2rad)
  hxci = hx*cosi
  hc_kwav = planck*c/wavang/1.e-10/boltz
  wavang2 = wavang*wavang
  rlold  = rlosch*rsch
  
  
  #print t0v,t0i
  
  #this estimate for the cutoff radius is based on the max radius of the
  #highest lag (re-arrange equation 3 in Starkey et al 2017)
  #should be ok but might exclude some significant low lags for a VERY hot, edge on disk
  rhisch = -1.* max((tauhi - hxci)/(1.+sini), tauhi) #if -ve then this is in light days
  #print rhisch
  if (rhisch < 0):
   rhild  = np.abs(rhisch)
  else:
   rhild  = rhisch*rsch
  
  
  
  
  
  av4   = alpha_visc*4 
  ai4   = alpha_irad*4 
  #now define radius grid be smart here.
  
  #use a cutoff x_stop to determine when to stop the radius grid
  dtau   = taus[1] - taus[0]
  rhilog = dtau
  rtemp = np.array([rhilog,10*rhilog]) 
  rtl   = np.log(rtemp)
  ttemp4 = t0v4*(rtemp)**av4 + t0i4*(rtemp)**ai4
  ttemp = np.sqrt(np.sqrt(ttemp4))
  ttl    = np.log(ttemp) 
  tstop  = hc_kwav/xstop
  grad = (rtl[1] - rtl[0]) / (ttl[1] - ttl[0])
  rhil = rtl[0] +  grad*( np.log(tstop) - ttl[0] )
  rhild   = np.exp(rhil)
  
  #rt4   = rtemp**4
  #ttemp4 = t0v4*(rtemp)**av4 + t0i4*(rtemp)**ai4
  #grad = (rt4[1] - rt4[0])/(ttemp4[1] - ttemp4[0])
  #t_stop4 = (hc_kwav/xstop)**4
  #rhild4 = rt4[0] + grad*(t_stop4 - ttemp4[0])
  #ttemp=ttemp4**0.25
  #xtemp = hc_kwav/ttemp
  #rhild4 = np.interp(t_stop4,ttemp4,rt4)
  #rhild = np.sqrt(np.sqrt(rhild4))
  #print rtemp
  #print ttemp
  #print xtemp
  #print xstop, tstop, rhild, grad, rtl, ttl
  #print rhild
  #print 'now defined a stopping radius e.g radius at which'
  #print 'temp give x =hc/(wav T) > xstop. xstop default is 15 now set up radius grid with this'
  #print 'upper radius BUT how can it be 2 light days because we need response at time delays',
  #print 'longer than this? FIX THIS' 
  #raw_input()
  #build grid
  rgridlog = np.exp( np.arange(np.log(rlold), np.log(rhilog), udlnr)[:-1] )#np.logspace(np.log(rlold),np.log(rhilog))
  rgridlin = np.arange(rhilog,rhild,rhilog)
  rgrid    = np.concatenate((rgridlog,rgridlin))
  nrad     = np.shape(rgrid)[0]
  dr       = rgrid[1:] - rgrid[:-1]
  
  
  
  
  #rgrid  = np.logspace(np.log10(rlold),np.log10(rhild),nrad)
  
 
  
  
  
  
  #calculate temperature at each radius grid
  tv4   = t0v4*(rgrid)**av4
  ti4   = t0i4*(rgrid)**ai4
  rir0b = ti4/t0i4 #this is just (r/r0)^alpha_irad
  ttot4 = tv4 + ti4
  ttot2 = np.sqrt(ttot4) 
  ttot  = np.sqrt(ttot2)
  ttot3 = ttot2*ttot
  
  #each point will have a time delay and a weighting append these to a list
  #for each point in the disc
  
  #loop of azimuths
  delsave = []
  wsave   = []
 
  
  if (quick == 1):
   for i in range(nrad-1):
    #quick way 
    ttotnow   = ttot[i]
    ttot3now  = ttot3[i]
    radlo = rgrid[i]
    radhi = rgrid[i+1]
    drad  = dr[i]#radhi - radlo
    
    #now azimuth grid
    azwidth = drad/radlo
    azgrid = np.arange(0.0, twopi,azwidth)
    naz = np.shape(azgrid)[0]
    nazsub1 = naz - 1
    azgrid_s = azgrid[1:] - azgrid[:-1]
    
    #print drad, radhi, azwidth,'sdfsd',radhi*azwidth
    #print radlo, radhi, azwidth, drad, naz,'sdfsf'
    
    raz   = np.random.uniform(radlo,radhi,nazsub1)   
    daz  = np.sqrt(raz*raz + hx2)
    az   = np.random.uniform(nazsub1)*azgrid_s + azgrid[:-1]#np.random.uniform(azgrid[:-1],azgrid[1:],1)
    caz  = np.cos(az)
    tdl  = hxci - raz*caz*sini + daz
    x    = hc_kwav/ttotnow#hc_kwav/ttot#hc_kwav/ttotnow
    
     
    #print radlo, x
    x2   = x*x
    #the radlo * drad *azwidth is the solid angle element
    #azwidth can be left off here as it is always the same
    weight = rir0b[i]/ttot3now * x2/wavang2/(np.cosh(x) - 1) * radlo*drad*azwidth                 #rir0b/ttot3 * x2/wavang2/(np.cosh(x) - 1) * radlo*drad*azwidth      #
    wsave.append([weight]*nazsub1)
    delsave.append(tdl)
    
  else:
   for i in range(nrad-1):
    radlo = rgrid[i]
    radhi = rgrid[i+1]
    drad  = radhi - radlo
    
    #now azimuth grid
    azwidth = drad/radhi
    azgrid = np.arange(0.0, twopi,azwidth)
    naz = np.shape(azgrid)[0]
    nazsub1 = naz - 1
    azgrid_s = azgrid[1:] - azgrid[:-1]
    
    raz   = np.random.uniform(radlo,radhi,nazsub1)
    daz  = np.sqrt(raz*raz + hx2)
    az   = np.random.uniform(nazsub1)*azgrid_s + azgrid[:-1]#np.random.uniform(azgrid[:-1],azgrid[1:],1)
    caz  = np.cos(az)
    tdl  = hxci - raz*caz*sini + daz
    tv4   = t0v4*(raz)**av4 #modification for inner radius to the right (negligible)* (1 - np.sqrt(rlold/raz)) / (1 - np.sqrt(rlold/1))
    ti4   = t0i4*(raz)**ai4
    rir0b = ti4/t0i4 #this is just (r/r0)^alpha_irad
    ttot4 = tv4 + ti4
    ttot2 = np.sqrt(ttot4) 
    ttot  = np.sqrt(ttot2)
    ttot3 = ttot2*ttot
    x      = hc_kwav/ttot
    x2     = x*x
    weight = rir0b/ttot3 * x2/wavang2/(np.cosh(x) - 1) * radlo*drad*azwidth
    wsave.append(weight)
    delsave.append(tdl)
 
   #tv4   = t0v4*(raz)**av4 #modification for inner radius to the right (negligible)* (1 - np.sqrt(rlold/raz)) / (1 - np.sqrt(rlold/1))
   #ti4   = t0i4*(raz)**ai4
   #rir0b = ti4/t0i4 #this is just (r/r0)^alpha_irad
   #ttot4 = tv4 + ti4
   #ttot2 = np.sqrt(ttot4) 
   #ttot  = np.sqrt(ttot2)
   #ttot3 = ttot2*ttot
   
   
   
   #introduce a lower threshold weight to abort the iterations (to save time)
   
  
  
  delsave = [item for sublist in delsave for item in sublist]
  wsave   = [item for sublist in wsave for item in sublist]
  delsave = np.array(delsave)
  wsave   = np.array(wsave)
  
  nds = np.shape(delsave)[0]
  
  
  
  #add smoothing function to approximate delta function
  #ntaus = np.shape(taus)[0]
  sigtau = 5*dtau/2
  sigtau2 = sigtau*sigtau
  sigtaulim = 5*sigtau
  
  
  print 'ok now it works but is slow'
  print 'need to find a numpy or scipy gaussian smoothing function'
  print 'ready made function wil be faster than anything I write down below'
  print 'was fine for fortran but python prefers ready made smoothing function'
  print 'google numpy or scipy gaussian filter 1d etc'
  #raw_input()
  #psis = scipy.ndimage.filters.gaussian_filter1d(, sigma, axis=-1, order=0, output=None, mode='reflect', cval=0.0, truncate=4.0)
  psis = np.zeros(ntaus)
  for id in range(nds):
   delnow = delsave[id]
   idlo = int( max(1,np.floor((delnow - sigtaulim)/dtau)) )
   
   #print 'dfsdf',ntaus, np.ceil((delnow + sigtaulim)/dtau), np.max(ntaus,np.ceil((delnow + sigtaulim)/dtau))
   idhi = int( min(ntaus,np.ceil((delnow + sigtaulim)/dtau)) )
   idinc = np.arange(idlo,idhi,1)
   #print idinc
   
   tdelsubtau = delnow - taus[idinc]
   gtemp = np.exp(-0.5*tdelsubtau*tdelsubtau/sigtau2)
   gsum = np.sum(gtemp)
   wnow  = wsave[id]
   psis[idinc] = psis[idinc] + wnow*gtemp/gsum 
  
 # for i in range(ntaus-1):
 #  taulo = taus[i]
 #  tauhi = taus[i+1]
 #  taulimlo = taulo - sigtaulim
 #  taulimhi = tauhi + sigtaulim
 #  taucent = (taulo + tauhi)/2
 #  idxsmooth = np.where((delsave >= taulimlo) & (delsave < taulimhi))[0]
 #  taunow    = delsave[idxsmooth]
 #  wnow      = wsave[idxsmooth]
 #  
 #  a = taunow - taucent
 #  b = sigtau
 #  aa_bb2 = a*a/b/b
 #  gauss     = np.exp(-aa_bb2/2)
 #  gsum      = np.sum(gauss)
 # 
 #
 #  #WONT SOME SURFACE ELEMENTS BE COUNTED TWICE AND SO BE INCLUDED IN MORE THAN ONE GAUSSIAN WEIGHTING? YES BUT I DONT THINK THIS MATTERS
 #  psis[i] = np.sum( wnow*gauss/gsum )
 #  
 #  
 #  #print taus[i], psis[i], ntaus
  #if norm == 1then normalise so max == 1
  if (norm == 1):
   pt   = (psis - np.min(psis))
   psis = pt/np.max(pt)
 
 return(psis)
#






#outputs are psis[ntau] to input set of time grid[ntau]
#make code to test and plot 
 
import timeit 
wavang = 4000.
embh   = 1.e8
emdot  = 1.0
deginc = 0.0

t0v   = -1.6e4
t0i   = -1.2e4
taus   = np.arange(0.0,30.,0.1)



start = timeit.default_timer()
psis   = pytfb_sub(taus,embh,emdot,wavang, deginc,t0vin = t0v, t0iin = t0i, norm = 1)
stop = timeit.default_timer()



print 'run time', stop-start
 
fig = plt.figure()
ax1 = fig.add_subplot(111)


ax1.plot(taus,psis)
plt.savefig('pytfbplottest_quick.pdf')



















###T(R) prof


import numpy as np
import const
from pylab import *

##Define the array containing the radius and angular information
size=1000
temp_inf=np.zeros((size,2,2))
#delta_r=(r_out-r_in)/size

##define the parameters
##default parameters
#delta_r=0.1
#r_out=1
#r_in=0.1
#inc=0
#D=1
#M=1*10**8
#M_dot=1
#R_star=3


##make the following code a definition
def flat_disk(delta_r,r_out,r_in,inc,D,M,M_dot,R_star):


	##set this to 1 if you want to specify more parameters


	##text for plots
	delta_r=(r_out-r_in)/size
	text=np.zeros(6)
	text[0]=D
	text[1]=r_in
	text[2]=r_out
	text[3]=M_dot
	text[4]=M
	text[5]=inc

	##black hole scwarzschild radius
	def r_sch(M):
		rs=2*const.G*M/const.c**2
		return(rs)


	M=M*const.Msun
	M_dot=M_dot*const.Msun
	D=D*const.c/const.H0
	delta_r=delta_r*const.ps
	r_out=r_out*const.ps
	r_in=r_in*const.ps
	R_0=r_sch(M)*R_star

	##populate the x and y dimensions of the array with radial and azimuthal coordinates 
	temp_inf[:,0,0]=1.*np.arange(size)/size * (r_out-r_in)  +r_in
	temp_inf[:,1,0]=1.*np.arange(size)/size * 2*np.pi
	#delta_r=(r_out-r_in)/size

	## solid angle  ## radii are stored in temp_inf[:,0,0]
	s_a=2*np.pi*temp_inf[:,0,0]*delta_r*np.cos(inc)/D**2



	##define the temperature radius profile for a steady state accretion disk in parsecs
	##viscous dissipation due gives rise to a factor of 3/8pi
	def T_R(M,M_dot,R,R_star):
		#M=M*const.Msun
		#M_dot=M_dot*const.Msun
		#R=R*const.ps
		#R_0=r_sch(M)*R_star
		T=(3*const.G*M*M_dot*(1-(R_0/R)**0.5)/(8*np.pi*const.sb*R**3))**0.25
	return(T)


	

	###Now need to multiply by the solid angle element of each annuli


	T=T_R(M,M_dot,temp_inf[:,0,0],R_star)	

	##
	#define an array which stores wavelengths from the wein temperature of the lowest radi to highest radi

	wavmin=const.w_c/T[0]
	wavmax=const.w_c/T[size-1]
	wavelength=1.*np.arange(size)/size * (wavmax-wavmin) +wavmin   ## store the wavelength ranges over to calculate disk spectrum 	
	
	##plank spectrum as a function of wavelength have to sum over temperature contributions from all of disk
	B_T=np.zeros(size)
	for i in range(size):
		B_T[i]=(2*const.plank*const.c**2/wavelength[i]**5 * (np.exp(const.plank*const.c/(wavelength[i]*const.k*T)) - 1)**-1 * s_a).sum()


	
	
	## Just to plot everything

	output=0
	if output == 1:
		plot(wavelength,B_T)
		xlabel('Wavelength / m')
		ylabel('flux Wm^-1')
		xticks(fontsize=15)
		yticks(fontsize=15)
		title ('Z = '+str(text[0])+' .  r_in/r_out (pc)=' +str(text[1])+'/'+str(text[2])+'.  M='+str(text[4])+'.  M dot='+str(text[3])+'.  Inc='+str(text[5]))
		show()
	



import numpy as np
import matplotlib.pyplot as plt
import matplotlib.ticker as mtick
import os
plt.rcParams['axes.linewidth'] = 2.0 #set the value globally
plt.rcParams['lines.linewidth'] = 2.0
plt.rcParams['axes.labelsize'] = 2.0



h    = 6.626e-34
c    = 2.9979e8
kb   = 1.38e-23
Mpc  = 3.0857e22
Mpc2 = Mpc*Mpc
ang  = 1.e-10
tday = 3600.*24
ld = c*tday





#calculate flux observed from a disk annulus at radius r T(r) = T0 (r/r0)^(-3/4)
#dr, r0 in light days
def myfnu(T,wavang,r_r0,T0, D_Mpc,degi = 0.0, dr = 0.1, r0 = 0.1):

 bnu_p = bnu(T,wavang)
 radi = np.pi/180. * degi
 cosi = np.cos(radi)
 
 d = D_Mpc*Mpc 
 a = 2*np.pi/(d*d) * dr * r_r0 *r0 * ld*ld
 op = bnu_p * a/(d*d)
 return(op)




#define plank function fnu
def bnu(T,wavang):
 
 wav = wavang*ang
 wav2 = wav*wav
 wav3 = wav2*wav
 a = 2*h*c/wav3
 
 b = 0
 if (T == 0):
  bnu = 0
 else:
  b = h*c/(wav*kb*T)
  bnu = a/ (np.exp(b) - 1.)
 #if (T != 0):
 # print 'bnu'
 # print bnu
 # print 'b'
 # print a
 # print 'c'
 # print b
 # print 'wavang'
 # print wavang
 # print 'temp'
 # print T
  #raw_input()
 return(bnu)
 
 
 
 

#make plots of planck function for a list of T[t1,t2,t3,..]
def plotbnu(T,wav,colplot=[],fname='plotbnu.png',ylo=0.0,ymax=[],fudge=[]):
 
 pwd=os.getcwd()
 nT = len(T)
 dir = './plotbnu_dir'
 
 if (fudge == []):
  fudge = [1.]*nT
 
 if (colplot==[]):
  colplot = ['b']*nT

 
#make directory to save bnu's in (remove old ones if present)
 if not os.path.exists(dir):
  os.makedirs(dir)
 
 os.chdir(dir)
 #os.system('rm *.png')
 
 
 fig = plt.figure()
 ax1 = fig.add_subplot(111)
 ax1.yaxis.set_major_formatter(mtick.FormatStrFormatter('%.2e'))
 ax1.xaxis.set_major_formatter(mtick.FormatStrFormatter('%d'))
 ax1.set_xlabel(r'$ \lambda / \AA $')
 ax1.set_ylabel(r'$f_{\nu} ( \lambda ) $')
 ax1.get_yaxis().set_ticks([])

 for i in range(nT):
  Tnow = T[i]
  specint = bnu(Tnow,wav)
  specint = specint/specint.max()*fudge[i]
  
  ax1.plot(wav,specint,color=colplot[i])
  
  #print ymax,'klljlksdjsl'
  if (ymax != []):
   ax1.set_ylim(ylo,ymax[i])
   print 'new ylim',i,Tnow,ymax[i]
  
  #fname = 'plotbnu_'+str(Tnow)+'.png'
  plt.savefig(fname)
  
  os.chdir(pwd)


## read a file and dont give me crappy errors
## ouptut numpy 2d array (nrows,ncols)
#alterd code to deal with lines with uneven number of lines returns rejected points in a fail lsit

import numpy as np



def myreadlines(fname):

 f = open(fname)
 
 faillist = []
 idx = 1
 a = []
 for i in f:
  #a.append(np.float(i.split()))
  b = []
  new = i.split()
  for idx2 in range(len(new)):
   #produce exception if there is a hidden character in there
   try:
    b.append(np.float(new[idx2]))
   except:
    print 'myreadlines.py: Cannot convert to float index',idx,idx2
    #print new
    #print len(new),new[0]
    b.append(np.nan)
  a.append(b)
  idx = idx+1
  #print f.readline()
 #a = np.array(a)
 #
 
 
 nrow = len(a)
 
 #how to deal with files that have an uneven number of columns (find the mode number of columnsa and put that info in array
 #skipping elements that dont fit
 ncollist = [len(a[i]) for i in range(nrow)]
 ncol = max(set(ncollist), key=ncollist.count)
 
 #ncol = len(a[0])
 
 b = np.zeros((nrow,ncol))
 idcount = 0
 for i in range(nrow):
  try:
   b[idcount,:] = a[i][:]
   idcount = idcount + 1
  except:
   print 'something wrong with row',i,'...skipping'
   faillist.append(a[i][:])
 b = b[:idcount,:]  
   
 f.close()
 return(b,faillist)
 




import numpy as np
import mydiptest_2 as md2
import myrandom as mr
import matplotlib.pylab as plt

nboot = 10
cent = np.arange(0,10,1)

pplot = []
dplot = []

xpdf1 = mr.normdis(5000,0,1)

#try with gaussian
ncen = np.shape(cent)[0]
for ib in range(ncen):
 xpdf2 = mr.normdis(5000,cent[ib],1)
 xpdf  = np.concatenate((xpdf1,xpdf2))
 print 'before dtest',ib
 dtest = md2.DipTest(xpdf)#md2.DipTestSig(xpdf,1)
 pplot.append(dtest[0])
 dplot.append(dtest[0])


fig = plt.figure()
ax1 = fig.add_subplot(211)
ax1.plot(cent,pplot)
ax1.set_ylabel('p value')

ax1 = fig.add_subplot(212)
ax1.plot(cent,dplot)
ax1.set_ylabel('dip test')


plt.savefig('fig_diptesttest.pdf')




import numpy as np
import matplotlib.pyplot as plt
from matplotlib.ticker import NullFormatter
import mymedquart


def myscatterhist(x,y,col='b',l='',xlab='',ylab='', xlo = 666,xhi = 666,ylo = 666,yhi = 666,nbins = 100):
 
 paperplot = 1
 print 'inside function', l
 nullfmt   = NullFormatter()         # no labels
 
 # definitions for the axes 
 left, width = 0.1, 0.25
 bottom, height = 0.1, 0.25
 bottom_h = left_h = left+width+0.02
 
 

 rect_scatter = [left, bottom, width, height]
 rect_histx = [left, bottom_h, width, 0.2]
 rect_histy = [left_h, bottom, 0.2, height]
 

 # start with a rectangular Figure
 #plt.figure(1, figsize=(8,8))
 
 axScatter = plt.axes(rect_scatter)
 axHistx = plt.axes(rect_histx)
 axHisty = plt.axes(rect_histy)
 
 # no labels
 axHistx.xaxis.set_major_formatter(nullfmt)
 axHisty.yaxis.set_major_formatter(nullfmt)
 
 # the scatter plot:
 axScatter.scatter(x, y, color = col, s=0.5)#np.ones(x.shape[0])*1)
 # add vertical lines to the plot to indicate the mean and sd of the parameter
 medloupx = mymedquart.mymedquart(x,0.68)
 lo = medloupx[1]
 up = medloupx[2]
 med = medloupx[0]
 #axScatter.vlines(med,ylo,yhi,color=col,linewidth=3)
 #axScatter.vlines(lo,ylo,yhi,color=col)
 #axScatter.vlines(up,ylo,yhi,color=col)
 
 medloupy = mymedquart.mymedquart(y,0.68)
 lo = medloupy[1]
 up = medloupy[2]
 med = medloupy[0]
 #axScatter.hlines(med,xlo,xhi,color=col,linewidth=3)
 #axScatter.hlines(lo,xlo,xhi,color=col)
 #axScatter.hlines(up,xlo,xhi,color=col)
 
 
 # now determine nice limits by hand:
 if (xlo == 666 & xhi == 666):
  xhi = x.max()
  xlo = x.min()
 
 if (ylo == 666 & yhi == 666):
  yhi = y.max()
  ylo = y.min()
 
 binwidthx = (xhi - xlo)/nbins
 binwidthy = (yhi - ylo)/nbins
 
 
 axScatter.set_xlim( (xlo, xhi) )
 axScatter.set_ylim( (ylo, yhi) )
 axScatter.set_xlabel(xlab)
 axScatter.set_ylabel(ylab)
 
 #we need a sensible way to define the number of bins. The routine automatically tries to include everything
 binsx = np.arange(xlo, xhi + binwidthx, binwidthx)
 binsy = np.arange(ylo, yhi + binwidthy, binwidthy)
 
 #print '10',binsx.shape[0],binsy.shape[0],xlo,xhi,ylo,yhi,binwidthx, binwidthy
 #raw_input()
 axHistx.hist(x, bins=binsx, color = col, label=l)
 #axHistx.legend(l)
 
 
 #print '11'
 axHisty.hist(y, bins=binsy, orientation='horizontal', color = col)
 
 axHistx.set_xlim( axScatter.get_xlim() )
 axHisty.set_ylim( axScatter.get_ylim() )

 return axHistx


col =['r','b','k','p','y']
#fig = plt.figure()

plt.figure(1, figsize=(8,8))
for i in range(1):
 #ax1 = fig.add_subplot(2,2,i+1)
 a = np.random.randn(1000) + 25*i
 b = np.random.randn(1000) + 2*i

 
 ax1 = myscatterhist(a,b,ifig,nalong,nvert)

plt.show()




#code to test a more javelin styke way of running cream
import numpy as np
import mylcgen as mg
from myrandom import *
import matplotlib.pylab as plt
import myredcisq as mrc


#generate test light curve and add noise
dat      = mg.mylcgen(tlo=0,thi=100,dt=2.0)
ndat     = np.shape(dat[:,0])[0]
datmean  = np.std(dat[:,1])
sig      = np.ones(ndat)/10*datmean
for i in range(ndat):
 dat[i,1] = normdis(1,dat[i,1],sig[i])[0]
y        = dat[:,1]
time     = dat[:,0]

tlo = np.min(time)
thi = np.max(time)
dt  = np.mean(time[1:]-time[:-1])

dtres = dt/2



#fakelc making done now do actual routine
tgrid = np.arange(tlo,thi+dtres,dtres)
ngrid = np.shape(tgrid)[0]
nits  = 1000



#make a fake light curve for each teration and compute a fit statistic for each
ymsum  = np.zeros(ngrid)
ym2sum = np.zeros(ngrid)
wsum   = 0
cisq0  = 0
cisqsave = []
for i in range(nits):
 #print i
 dat      = mg.mylcgen(tlo=tlo,thi=thi,dt=dtres)
 cisq     = mrc.myredcisq(time,y,sig,dat[:,0],dat[:,1],npar = 0)[0]
 if (i == 0):
  cisq0 = cisq
 
 cisqsave.append(cisq)
 print i, cisq, cisq0
 w        = cisq#np.exp(-(cisq-cisq0)/2)
 wsum     = wsum   + w
 ymsum    = ymsum  + w*dat[:,1]
 ym2sum   = ym2sum + w*dat[:,1]**2

yplot   = ymsum / wsum / nits
ysdplot = (ym2sum - ymsum) / wsum / nits










#plot the results
fig = plt.figure()
ax1 = fig.add_subplot(111)
ax1.plot(tgrid,yplot,label=None,color='b')
ax1.errorbar(time,y,sig,color='k',label=None,ls='')
plt.savefig('test_j2.pdf')







### Script to plot an irradiated disk spectrum for a static disk

import numpy as np
import cgs
from pylab import *
from scipy import *
from plank import *


####Define all the default paramters

r_in = 3        ## ISCO radius in schwarzschild radii
r_out = 1e5     ## desired outer radius of accretion disk 
delta_r = 10  ## desired radial resolution 
M = 1e8         ## Black hole mass (solar masses)
D = 1 			## redshift distance to black hole
inc = 0 ## desired inclination (radians)
M_dot = 1 		## Accretion rate (solar masses per year)
beta = 0
up_wav  = 1e-6
low_wav = 1e-9
delta_wav = 1e-9

## Convert to SI units
M = M*const.Msun
rs = const.r_sch(M)
r_in = r_in*rs
r_out = r_out*rs
delta_r = delta_r*rs
M_dot=M_dot * const.Msun
D=D*const.c/const.H0




##define the temperature radius profile for a steady state accretion disk in parsecs
##viscous dissipation due gives rise to a factor of 3/8pi
def T_R(M,M_dot,R,R_in):
	T=(3*const.G*M*M_dot*(1-(R_in/R)**0.5)/(8*np.pi*const.sb*R**3))**0.25
	return(T)

	
## flat disk spectrum irradiated
def T_D(A,lum,H0,x):
	T_D = (1-A)*lum/(4*np.pi*const.sb*(x**2+H**2)) * x/(x**2 + H**2)**0.5
	return(T_D)

	
## plank spectrum (frequency) but in terms of wavelength. given wavelgnth, and temperature

def btu(temp,wave):
	plank=2*const.plank*const.c/wave**3 * (np.exp(const.plank*const.c/(wave*const.k*temp)) - 1)**-1
	return(plank)
	


### solid angle flat disk

def sa_flat(r,D,inc,delta_r):
	s_a=2*np.pi*r*delta_r*np.cos(inc)/D**2	
	return(s_a)
	


### Luminosity of the central source (powered by acrretion. May need to introduce efficiency
##parameter later
Lum = const.G * M * M_dot / r_in

##populate the x and y dimensions of the array with radial and azimuthal coordinates 

rad_inf=1.*np.arange(np.ceil((r_out-r_in)/delta_r))/np.ceil((r_out-r_in)/delta_r)*(r_out-r_in) + r_in


	

### Obtain the temperature radius relation, wavelengths of the corresponding peak wavelengths
## and solid angle element of each radii
#s_a=2*np.pi*rad_inf*delta_r*np.cos(inc)/D**2
#T=T_R(M,M_dot,rad_inf,r_in)	
#wavelength = const.w_c/T
wavelength = 1.*np.arange(np.ceil((up_wav-low_wav)/delta_wav))/np.ceil((up_wav-low_wav)/delta_wav)*(up_wav-low_wav) + low_wav








import numpy as np
import matplotlib.pylab as plt



#iteratively fit polynomials up to order, using BIC,AIC and reduced chi_squared to determine 
#the optimum number of parameters.
def fit_search(x,y,maxorder=8,xgrid=[]):
 
 
 
 yg_med   = []
 yg_lo    = []
 yg_hi    = []
 cov      = []
 cisq     = []
 cisq_red = []
 bic      = []
 aic      = []
 
 sig = 0.*x + 1.
 for i in range(1,maxorder):
  a = fit(x,y,sig,order=i,xgrid=xgrid)
  yg_med.append(a[0])
  yg_lo.append(a[1])
  yg_hi.append(a[2])
  cov.append(a[3])
  cisq.append(a[4])
  cisq_red.append(a[5])
  bic.append(a[6])
  aic.append(a[7])

 #sort by increasing fit metrics
 id_aic     = np.argsort(aic)
 id_bic     = np.argsort(bic)
 id_cisqred = np.argsort(np.abs(np.array(cisq_red) - 1.0))#np.argsort(cisq_red)
           
 print('best cisqred fit order:',id_cisqred[0])
 print('best aic fit order:',id_aic[0])
 print('best bic fit order:',id_bic[0])
 return(id_cisqred[0]+1,id_aic[0]+1,id_bic[0]+1)














#fit polynomial of a given order conf=0.05 for 95 % , 0.3173 for 1 sigma
def fit(x,y,sig=0,order=3,xgrid=[],confidence=0.3173,nits=20000,figure_title=''):

 #evaluate model on arbitrary time grid
 if (xgrid != []):
  xg = np.array(xgrid)
 else:
  xg = np.array(x)
 nxg = np.shape(xg)[0]

 #cannot fit trend with fewer points than polynomial order
 nx = np.shape(x)[0]
 oi = min(nx-4,order)
 if (oi < 0): 
  print('fit is degenerate, just using straight line')
  try:
   yg_med = np.ones(nxg)*y[0]
  except:
   yg_med = np.zeros(nxg)
  yg_lo  = yg_med*1
  yg_hi  = yg_med*1
  cov    = np.zeros((1,1))
  disq   = 0
  cisq = 0
  cisq_red = 0
  bic = 0
  aic = 0
  rmd = 0
  return(yg_med,yg_lo,yg_hi,cov,cisq,cisq_red,bic,aic,rmd)

 #if sig = [] then dont know error bars assign equal weight
 if (type(sig) == np.ndarray):
  s = 1.*sig
  noerrors = 0
 else:
  s = np.ones(nx)
  noerrors = 1
 
 
 

 #compute and subtract pivot point (get better fit with reduced correlated parameter uncertainties)
 xp = np.sum(x/s**2)/np.sum(1./s**2)
 
 
 coefs,cov = np.polyfit(x - xp,y,w=1./s,deg = oi,full=False,cov=True)
 
 if (noerrors == 1):
  
  ymod_itp = np.sum(np.array([coefs[-1-ip]*(x-xp)**ip for ip in range(oi+1)]),axis=0)
  
  print(np.shape(ymod_itp))
  #input()
  
  var =   np.sum((ymod_itp - y)**2)/nx
  
  #print('var',var)
  #cov = cov*var
 
 

 #monte carlo sample error snake
 c_mvn  = np.random.multivariate_normal(coefs,cov,size=nits)
 yg = np.zeros((nxg,nits))
 
 
 
 xgxp_grid = np.array([(xg-xp)**ip for ip in range(oi+1)])

 
 
 for iteration in range(nits):
  cnow = c_mvn[iteration,:]
  yg_temp = np.dot(xgxp_grid.T,cnow[::-1]) 
  #yg_temp = np.sum( np.array([cnow[order-ip]*(xg-xp)**ip for ip in range(order+1)] ), axis = 0)  

  yg[:,iteration] = yg_temp


 #compute final model and uncertainties
 #ygrid = np.mean(yg,axis=1)
 
 pc = [confidence/2 *100, 50. ,(1.-confidence/2)*100 ]
 yg_med_conf = np.percentile(yg,pc,axis=1).T
 
 yg_lo  = yg_med_conf[:,0]
 yg_med = yg_med_conf[:,1]
 yg_hi  = yg_med_conf[:,2]
   
 

 #interpolate model onto data
 yg_itp  = np.interp(x,xg,yg_med)
 
 
 
 
 
 

 
 #modify uncertainty to suit confidence
 ymed    = np.median(yg_med)
 diffs   = y - yg_itp
 id_hi     = np.where(diffs > 0)[0]
 diff_hi   = np.sort(diffs[id_hi])
 nhi       = np.shape(diff_hi)[0] 
 try:
  frac_hi   = diff_hi[np.int(nhi*(1.-confidence))] / np.median(yg_hi - yg_med)
 except:
  frac_hi = 1
 
 yg_hi_mod = (yg_hi - yg_med)*frac_hi + yg_med
 
 
 
 # print(nhi,'herere',order)
 # print(coefs)
 #
 # for it in range(np.shape(yg_med)[0]):
 #  print(xg[it],yg_lo[it],yg_med[it],yg_hi[it])
 # print()
 # for it in range(np.shape(yg_itp)[0]):
 #  print(x[it],y[it],yg_itp[it])  




 
 
 
 #for i in range(np.shape(yg_lo)[0]):
 # print(i,xg[i],yg_lo[i],yg_med[i],yg_hi[i])
 #print('cov')
 #print(cov)
 #print('pars')
 #print(coefs)
 #print('mvg')
 #print(c_mvn)
 #
 #print(frac_hi,diff_hi[np.int(nhi*(1.-confidence))],np.median(yg_hi - yg_med))
 #print()
 
 
 
 id_lo     = np.where(diffs < 0)[0]
 diff_lo   = np.sort(np.abs(diffs[id_lo]))
 nlo       = np.shape(diff_lo)[0]
 try:
  frac_lo   = diff_lo[np.int(nlo*(1.-confidence))] / np.median(yg_med - yg_lo)
 except:
  frac_lo = 1.0
 yg_lo_mod = yg_med - (yg_med-yg_lo)*frac_lo
 
 #print(frac_lo,diff_lo[np.int(nlo*(confidence))],np.median(yg_med - yg_lo))
 #print() 
 #input()
  
  
  
  
 #absdiff_sort = np.sort(np.abs(diff))#np.where(np.abs(diff)
 #frac = absdiff_sort[np.int( nxg*(1.-confidence) )]/absdiff_sort[np.int(nxg/2)]
 
  
  
 #interpolate model uncertainties onto data
 hi_itp  = np.interp(x,xg,yg_hi_mod)
 lo_itp  = np.interp(x,xg,yg_lo_mod)
 sig_itp = (hi_itp - lo_itp)/2
 
 
 #compute model evaluation stats (big numbers for lots of parameters or poor fits)
 #reduced chisquared
 cisq = np.sum(((yg_itp - y)/sig_itp)**2)
 ndof = nx - oi + 1
 cisq_red = cisq/ndof
 
 #autocoorelation
 rmd = np.corrcoef(yg_itp,y)[0,1]
 
 #Bayesian information criterion 
 bic = cisq + (oi+1)*np.log(nx)
 
 
 #Akaike information criterion
 aic = cisq + 2*(oi+1)
 
 
 #results summary
 print('fit results for poilynomial order: ',oi)
 print('')
 print('Smaller numbers better')
 print('reduced chisquare: ',cisq_red)
 print('aic: ',aic)
 print('bic: ',bic)
 print('chisquare: ',cisq)
 print('correlation coefficient: ',rmd)
 print()
 print()
 

 #fig = plt.figure()
 #ax1 = fig.add_subplot(111)
 #ax1.scatter(x,y)
 #ax1.plot(xg,yg_med,label='model')
 #ax1.plot(x,yg_itp,label='interpolated')
 #ax1.fill_between(xg,yg_lo,yg_hi,alpha=0.3,label='old uncertainties')
 #ax1.fill_between(xg,yg_lo_mod,yg_hi_mod,alpha=0.3,label='new uncertainties')
 #plt.legend()
 #plt.show()
 
 

 if (figure_title != ''):
  fig = plt.figure()
  ax1 = fig.add_subplot(111)
  ax1.scatter(x,y)
  ax1.plot(xg,yg_med,label='model')
  ax1.plot(x,yg_itp,label='interpolated')
  ax1.fill_between(xg,yg_lo,yg_hi,alpha=0.3,label='old uncertainties')
  ax1.fill_between(xg,yg_lo_mod,yg_hi_mod,alpha=0.3,label='new uncertainties')
  plt.legend()
  if (figure_title == 'show'):
   plt.show()
  else:
   plt.savefig(figure_title)
  
 return(yg_med,yg_lo,yg_hi,cov,cisq,cisq_red,bic,aic,rmd)
 
 
 
 
 
 
 
 
 
 
##test with fake data
#
#import numpy as np
#import matplotlib.pylab as plt
#import glob
#import pandas as pd
#import datetime
#import statsmodels.api as sm
#
#combine = 0
#frequency = 1./30. #fake 6 month signal
#f2 = 1./60
#tlo = 0.0
#thi = 365
#tref = 0.0#thi/2
#dt = 1.0
#lab = ''
#amp1 = 5.0
#amp2 = 10.0
#amps = [amp1,amp2]
#freqs = [frequency,f2]
#poly = [0,0,5.e-5,0.0]
#tfclo = thi
#tfchi = thi + 365
##generate synthetic time series for test
#t = np.arange(tlo,thi,dt)
#x = 5*np.sin(2 * np.pi * frequency * t) + 10*np.sin(2 * np.pi * f2/2 * t)
##add trend final -10.0 indicates amplitude at the end of the time sequence
#grad = 1.0
#y1   = 10.0
#noiseamp = 0.5
#
#pc_forecast = 0.4 #forecast ahead 40% the original length of the time series
#
##specify the confidence interval with the alpha argument 
##e.g alpha = 0.05 is the 95pc confidence interval (2 sigma)
##alpha = 0.32 is the 1 sigma confidence interval (68pc)
#alpha_conf = 0.32#0.05
#
#
#
#def trend(t,poly,tref,freqs,amps):
# nt = np.shape(t)[0]
# npoly = len(poly)
# namps = len(amps)
# xtot = []
# for i in range(nt):
#  trend = np.sum([poly[ip]*(t[i]-tref)**ip for ip in range(npoly)] )
#  seasonal = np.sum([amps[ip]*np.sin(2*np.pi*freqs[ip]*t[i]) for ip in range(namps)] )
#  xtot.append(trend + seasonal)
#
# return(np.array(xtot))
# 
# 
#

#signal = trend(t,poly,tref,freqs,amps) 
##grad*(x-t[-1]) + y1
#nt = np.shape(t)[0]
#noise = np.random.randn(nt)*noiseamp
#signal = signal + noise
#
#t = np.arange(100)
#x = np.random.randn(100)*1.0 + 5.0 #+ 2.3*t + 3*t**2
#
#
#tgrid = np.arange(-20,120,0.1)
#fit_search(t,x,maxorder=8,xgrid=tgrid)

##signal = trend(t,poly,tref,freqs,amps) 
###grad*(x-t[-1]) + y1
##nt = np.shape(t)[0]
##noise = np.random.randn(nt)*noiseamp
##signal = signal + noise
##
##t = np.arange(100)
##x = np.random.randn(100)*1.0 + 5.0 #+ 2.3*t + 3*t**2
##
##
##tgrid = np.arange(-20,120,0.1)
##fit_search(t,x,maxorder=8,xgrid=tgrid)



#calculates the black body spectrum of an accretion disc
#by default at 70 Mpc in mJy (if 1 else if mjy = 0 use erg/s/ang/cm^2)

#INPUT wav[...nwav] list of wavelengths to evaluate spectrum
#..... embh,emdot (the black hole mass and acretion rate in solar masses and solarmasses per year)
#..... deginc the inclination of the disc, 0 = face-on
#..... dl (optional) the luminosity distance to the disc in Mpc (default is 70)
#..... radlosch, radhisch, ngrid control the resolution of the disc grid (only change if having problems)
#..... mjy = 1 if want spectrum in mjy else get in erg/s/ang/cm^2
#
#OUTPUT mds[...nwav] spectrum
  
import numpy as np
import matplotlib.pylab as plt
import astropy_stark.mytemp0 as mt0
import astropy_stark.myplank as mp
twopi = 2*np.pi
deg2rad = twopi/360.




def mds(wav,embh,emdot,degi, dl = 70., radlosch=3.0,radhisch=10000., ngrid = 1000, mjy = 1):
 r0 = 1.0
 sb      = 5.670367e-8
 gnewt   = 6.67408e-11
 msun    = 1.98855e30
 secyr   = 31557600.0
 ld      = 2.59020683712e+13
 c       = 299792458.0
 rinld   = radlosch * 2*gnewt*msun*embh/c/c/ld
 if (rinld > r0):
  r0 = np.ceil(rinld/r0)
  #print('mydisksim.py: black hole mass too big for inner radius of 1 light day... expanding to',r0)

 cosi  = np.cos(deg2rad*degi)
 ldMpc = 1191286169.529
 ldMpc2 = ldMpc*ldMpc
 rsch   = 1.15821e-10*embh #
 radlo  = radlosch*rsch
 radhi  = radhisch*rsch
 rgrid  = np.logspace(np.log10(radlo),np.log10(radhi),ngrid)
 drgrid = rgrid[1:] - rgrid[:-1]
 drgrid = np.append(drgrid,drgrid[-1])
 #calculate appropriate temperatures for this embh and emdot
 t0v = mt0.tv0(embh,emdot,r0=r0,rinsch=radlosch)
 t0i = mt0.ti0(embh,emdot,r0=r0,hxsch=3,eta = 0.1)
 tr  = mt0.tr(rgrid,t0v,t0i,embh, r0=r0,alpha_visc=-0.75,alpha_irad=-0.75,rinsch = radlosch)
 
 
 #calculate the planck function for this radial grid
 fnu = []
 for wavnow in wav:
  
  if (wavnow == 0):
   fnu.append(0)
  else:
   bnu = mp.bnuvec(wavnow,tr,mjy=mjy)
   weight = rgrid * drgrid * bnu
   fnu.append( np.sum(weight)*twopi/dl/dl/ldMpc2 * cosi )
 
 return(fnu) 
 
 
 
 
#test the code
#
#dl = 70.0
#embh = 3.e9
#emdot = 100.
#wav = np.logspace(2.,6,1000)
#nrgrid = 1000
#degi = 0.0
#radlosch = 3.0
#radhisch = 10000.0 
#
#
#fnu = mds(wav,embh,emdot,degi, dl = 70., radlosch=3.0,radhisch=10000., ngrid = 1000, mjy = 1) 
##test diag
##for i in range(nrgrid):
## print rgrid[i], drgrid[i], bnu[i] 
# 
##test plot
#
#fig = plt.figure()
#ax1 = fig.add_subplot(111)
#ax1.plot(wav, fnu)
#ax1.set_xscale('log')
#ax1.set_yscale('log')
#plt.savefig('mydisksim_plot.pdf')
#
#
#
##test tr
##fig = plt.figure()
##ax1 = fig.add_subplot(111)
##ax1.plot(rgrid, tr)
##ax1.set_xscale('log')
##ax1.set_yscale('log')
##plt.savefig('mydisksimtr_plot.pdf')
##




 




import matplotlib.pyplot as plt
from matplotlib.patches import Ellipse
import numpy as np

def mce(cov, pos, nstd=2, ax=None, **kwargs):
    """
    Plots an `nstd` sigma error ellipse based on the specified covariance
    matrix (`cov`). Additional keyword arguments are passed on to the 
    ellipse patch artist.

    Parameters
    ----------
        cov : The 2x2 covariance matrix to base the ellipse on
        pos : The location of the center of the ellipse. Expects a 2-element
            sequence of [x0, y0].
        nstd : The radius of the ellipse in numbers of standard deviations.
            Defaults to 2 standard deviations.
        ax : The axis that the ellipse will be plotted on. Defaults to the 
            current axis.
        Additional keyword arguments are pass on to the ellipse patch.

    Returns
    -------
        A matplotlib ellipse artist
    """
    
    def eigsorted(cov):
        vals, vecs = np.linalg.eigh(cov)
        order = vals.argsort()[::-1]
        return vals[order], vecs[:,order]

    if ax is None:
        ax = plt.gca()

    vals, vecs = eigsorted(cov)
    theta = np.degrees(np.arctan2(*vecs[:,0][::-1]))

    # Width and height are "full" widths, not radius
    width, height = 2 * nstd * np.sqrt(vals)
    ellip = Ellipse(xy=pos, width=width, height=height, angle=theta, **kwargs)

    ax.add_artist(ellip)
    return ellip
    
    
    
    
def mc_b(cov, pos, nstd=2, ax=None, **kwargs):

 return mce(cov, pos, nstd=nstd, ax=ax)


#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!
#configuration for the fake data plotting routine for paper in myplot_results.py#!!!!!!!!!
#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!

dirs = ['/Users/ds207/Documents/standrews/sta/fort/fortcode/lucky_results/sdss_fake_new3/output_20180125_001',
'/Users/ds207/Documents/standrews/sta/fort/fortcode/lucky_results/sdss_fake_new10/output_20180125_001',
'/Users/ds207/Documents/standrews/sta/fort/fortcode/lucky_results/sdss_fake_new19/output_20180125_001']
truths = [np.log10(0.766459),np.cos(30.*np.pi/180),0.75]
col = ['r','k','b']
binsin = 20
fparms = 'outputpars.dat'
emlog  = 8.4971866218073941
bfrac = 3./4#10./12#2./3
idmbh = 2
iddeg = 3
idslope = 4












#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!
#!!!!!!!!!!!configuration for the fake data plotting routine in py_plot_fakedata.py#!!!!!!!!!
#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!#!!!!!!!!!

idburnin = 3./4

#light curve ascii files from which to calculate snr (x-axis of plot)
filesnr=['','','']

#light curve ascii files from which to calculate the inferred parameters (mdot, inc, trslope)
fileparm=['','','']


#files saving the response function
fileresp_mod = ['','','']
fileresp_sig = ['','','']

wavlag = ['g','i']
lagtrue = [1.0,1.3]


#indexes of columns in filedat parameter correspondong to each disk parameter
idparm = [1,2,3]
labparm = ['mdot','inc','slope']



import numpy as np
import numpy.random
import matplotlib.pyplot as plt

# Generate some test data
x = np.random.randn(8873)
y = np.random.randn(8873)

heatmap, xedges, yedges = np.histogram2d(x, y, bins=50)
extent = [xedges[0], xedges[-1], yedges[0], yedges[-1]]

plt.clf()
plt.imshow(heatmap, extent=extent)
plt.show()